# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import logging

from pysvnmanager.lib.base import *
from pysvnmanager.lib.text import to_unicode
from pysvnmanager.model.svnauthz import *
from pylons.i18n import _, ungettext, N_

log = logging.getLogger(__name__)

class RoleController(BaseController):
    requires_auth = True

    def __init__(self):
        c.menu_active = "role"
        try:
            self.authz = SvnAuthz(cfg.authz_file)
            self.login_as = session.get('user')
            # Used as checked in user to rcs file.
            self.authz.login_as = self.login_as
            self.aliaslist  = map(lambda x:x.uname, self.authz.aliaslist)
            self.userlist = map(lambda x:x.uname, self.authz.userlist)
            self.grouplist = map(lambda x:x.uname, self.authz.grouplist)
            self.is_super_user = self.authz.is_super_user(self.login_as)
            self.own_reposlist = self.authz.get_manageable_repos_list(self.login_as)
        except Exception, e:
            import traceback
            g.catch_e = [unicode(e), traceback.format_exc(5) ]
            return

    def __before__(self, action=None):
        super(RoleController, self).__before__(action)
        if not self.own_reposlist and not self.is_super_user:
            return redirect(url(controller='security', action='failed'))
        diff = self.authz.differ()
        if diff:
            c.global_message = _('Some one maybe you, has modified the svn authz file by hands. Please save once to fix possible config error.') + "<blockquote>" + "<br>".join(diff.splitlines()) + "</blockquote>"

    def index(self):
        c.revision = self.authz.version
        c.aliaslist  = self.aliaslist
        c.userlist = self.userlist
        c.grouplist = self.grouplist
        c.is_super_user = self.is_super_user
        return render('/role/index.mako')
    
    def get_role_info(self, role=None):
        members_count = 0;
        msg = ''
        if not role:
            d = request.params
            role = d.get('role')

        # get javascript code for top_form's role_selector
        if not role:
            msg += 'id[0]="%s";' % '...'
            msg += 'name[0]="%s";\n' % _("Please choose...")
            members_count += 1;
            for uname in self.grouplist:
                if uname == '*' or uname[0] == '$':
                    continue
                msg += 'id[%d]="%s";' % (members_count, uname)
                assert uname[0] == '@'
                msg += 'name[%d]="%s";\n' % (members_count, _("Group:")+uname[1:])
                members_count += 1;
            for uname in self.aliaslist:
                msg += 'id[%d]="%s";' % (members_count, uname)
                assert uname[0] == '&'
                msg += 'name[%d]="%s";\n' % (members_count, _("Alias:")+uname[1:])
                members_count += 1;
            msg += 'members_count=%d;\n' % members_count

        else:
            roleobj = self.authz.get_userobj(role)

            # get javascript code for g_form's group_member_list
            if roleobj and role[0] == '@':
                for i in roleobj:
                    uname = i.uname
                    #if uname == '*' or uname[0] == '$':
                    #    continue
                    msg += 'id[%d]="%s";' % (members_count, uname)
                    if uname[0] == '@':
                        msg += 'name[%d]="%s";\n' % (members_count, _("Group:")+uname[1:])
                    elif uname[0] == '&':
                        msg += 'name[%d]="%s";\n' % (members_count, _("Alias:")+uname[1:])
                    else:
                        msg += 'name[%d]="%s";\n' % (members_count, uname)
                    members_count += 1;
                msg += 'members_count=%d;\n' % members_count

            # get javascript code for a_form's alias as username
            elif roleobj and role[0] == '&':
                msg +='aliasname = "%s";' % roleobj.uname
                msg +='username = "%s";\n' % roleobj.username
        
        msg += 'revision="%s";\n' % self.authz.version
        return msg
        
    def save_group(self):
        assert self.is_super_user
        d = request.params
        member_list = []
        msg = ""

        rolename = d.get('rolename')
        autodrop = d.get('autodrop', 'no')
        members  = d.get('members', '')
        revision  = d.get('revision', self.authz.version)

        if autodrop.lower()=='yes':
            autodrop = True
        else:
            autodrop = False

        member_list.extend(map(lambda x: x.strip(), members.split(',')))
        
        log_message = _(u"User %(user)s changed group: %(grp)s. (rev:%(rev)s)") % \
                         {'user':session.get('user'), 'grp': rolename, 'rev': revision}

        try:
            self.authz.set_group(rolename, member_list, autodrop=autodrop)
            self.authz.save(revision, comment=log_message)
        except Exception, e:
            msg = to_unicode(e)

        log.info(log_message)
        if msg: log.error(msg)
        
        return msg
    
    def delete_group(self):
        assert self.is_super_user
        d = request.params
        rolename = d.get('role')
        revision  = d.get('revision', self.authz.version)
        msg = ''
        
        log_message = _(u"User %(user)s delete group: %(grp)s. (rev:%(rev)s)") % \
                         {'user':session.get('user'), 'grp': rolename, 'rev': revision}

        if rolename:
            try:
                self.authz.del_group(rolename)
                self.authz.save(revision, comment=log_message)
            except Exception, e:
                msg = to_unicode(e)

        log.info(log_message)
        if msg: log.error(msg)

        return msg
        
    def save_alias(self):
        assert self.is_super_user
        d = request.params
        aliasname = d.get('aliasname')
        username = d.get('username')
        revision  = d.get('revision', self.authz.version)
        msg = ""
        
        log_message = _(u"User %(user)s changed alias: %(alias)s. (rev:%(rev)s)") % \
                     {'user':session.get('user'), 'alias': aliasname, 'rev': revision}

        try:
            self.authz.add_alias(aliasname, username)
            self.authz.save(revision, comment=log_message)
        except Exception, e:
            msg = to_unicode(e)

        log.info(log_message)
        if msg: log.error(msg)
        
        return msg
    
    def delete_alias(self):
        assert self.is_super_user
        d = request.params
        aliasname = d.get('aliasname')
        revision  = d.get('revision', self.authz.version)
        msg = ''
        
        log_message = _(u"User %(user)s delete alias: %(alias)s. (rev:%(rev)s,%(msg)s)") % \
                 {'user':session.get('user'), 'alias': aliasname, 'rev': revision, 'msg': msg}

        if aliasname:
            try:
                self.authz.del_alias(aliasname)
                self.authz.save(revision, comment=log_message)
            except Exception, e:
                msg = to_unicode(e)

        log.info(log_message)
        if msg: log.error(msg)

        return msg

        



## -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from pysvnmanager.tests import *
import pylons.test

class TestLoginController(TestController):

    def test_login_logout(self):
        wsgiapp = pylons.test.pylonsapp
        config = wsgiapp.config
        params={}
        # login successful
        params['username'] = 'root'
        d = eval(config.get('test_users', {}))
        password = d.get(params['username'],'')
        params['password'] = password
        res = self.app.get(url(controller='security', action='submit'), params)
        self.assert_(res.status == "302 Found", res.status)
        self.assert_(res.location == '/' or res.location == 'http://localhost/', res.location)
        self.assert_(res.session['user'] == 'root', res.session)
        
        # keep session test
        res = self.app.get(url(controller='security', action='index'))
        self.assert_('<h2>Login</h2>' in res.body, res.body)
        self.assert_(res.session['user'] == 'root', res.session)
                                   
        # login with wrong password
        params['username'] = 'root'
        params['password'] = 'wrong_passwd'
        res = self.app.get(url(controller='security', action='submit'), params)
        self.assert_(res.status == "200 OK", res.status)
        self.assert_('Login failed for user: root' in res.body, res.body)
        self.assert_(res.session.get('user') == None, res.session.get('user'))
        
        self.login('jiangxin')
        res = self.app.get(url(controller='security', action='index'))
        self.assert_(res.session.get('user') == 'jiangxin', res.session)

        # logout
        res = self.app.get(url(controller='security', action='logout'))
        self.assert_(res.status == "302 Found", res.status)
        self.assert_(res.location == '/login' or res.location == 'http://localhost/login', res.location)
        self.assert_(res.session.get('user') == None, res.session.get('user'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from docutils import core
from docutils.writers.html4css1 import Writer,HTMLTranslator

class NoHeaderHTMLTranslator(HTMLTranslator):
    def __init__(self, document):
        HTMLTranslator.__init__(self,document)
        self.head_prefix = ['','','','','']
        self.body_prefix = []
        self.body_suffix = []
        self.stylesheet = []
    
    def astext(self):
        return ''.join(self.body)


_w = Writer()
_w.translator_class = NoHeaderHTMLTranslator

def reSTify(string):
    result = core.publish_string(string,writer=_w)
    if isinstance(result, basestring):
        result = unicode(result, 'utf-8')
    return result



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, os, string

SVNLOOK='/opt/svn/bin/svnlook'
MESSAGE="""
Dear {AUTHOR}:

We're sorry, but we just couldn't allow you to have the
revision {REVISION} commit.

       -- Love, Your Administrator(s).
"""

if len(sys.argv) < 5:
    sys.stderr.write(
        "Usage: %s REPOS AUTHOR BLOCKED_REV BLOCKED_AUTHOR [...]\n"
        "\n"
        "Disallow a set BLOCKED_AUTHORS from committing the revision\n"
        "expected to bring REPOS to a youngest revision of BLOCKED_REV.\n"
        "Written as a prank for use as a start-commit hook (which provides\n"
        "REPOS and AUTHOR for you).\n"
        "\n"
        "NOTE: There is a small chance that while HEAD is BLOCKED_REV - 2,\n"
        "a commit could slip in between the time we query the youngest\n"
        "revision and the time this commit-in-progress actually occurs.\n"
        "\n"
        % sys.argv[0])
    sys.exit(1)

repos = sys.argv[1]
author = sys.argv[2]
blocked_rev = sys.argv[3]
blocked_authors = sys.argv[4:]

if author in blocked_authors:
    youngest_cmd = '%s youngest %s' % (SVNLOOK, repos)
    youngest = os.popen(youngest_cmd, 'r').readline().rstrip('\n')

    # See if this is the blocked revision
    if int(youngest) == int(blocked_rev) - 1:
        MESSAGE = MESSAGE.replace('{AUTHOR}', author)
        MESSAGE = MESSAGE.replace('{REVISION}', blocked_rev)
        sys.stderr.write(MESSAGE)
        sys.exit(1)

sys.exit(0)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import os
import sys
sys.path.insert(0,os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from pysvnmanager.tests import *
from pysvnmanager import model
from pysvnmanager.model.svnauthz import *
import StringIO
from pprint import pprint

class TestModels(TestController):
    
    def __init__(self, *args):
        self.authz = SvnAuthz()
        self.f = StringIO.StringIO()
        super(TestModels, self).__init__(*args)

    def setUp(self):

        buff = '''
# version = 0.1.1
# admin : / = root, &admin
# admin : repos1 = @admin
# admin : repos2 = admin2
# admin : repos3 = admin3
# admin : reposx = 
# admin : 版本库1 = @管理组1

[groups]
admins=&admin,&007
admin=&admin, admin1, admin2, admin3
team1=user1,user11, @team2
team2=user2,user22,@team3,
# Wrong configuration: cyclic dependancies
#   team1->team2->team3->team1->...
# We can detect and fix.
team3=user3,user33,@team1
all=@team1,user3,user4
组1=&别名1, 用户2
管理组1=&别名1, 蒋鑫

[aliases]
admin=jiangxin
007=james
别名1=用户1

[repos1:/trunk/src]
user1=
&007 = r

[/trunk/src]
user1=r
user2=r

[repos1:/trunk]
user1=r
user2=

[/trunk]
user2=

[repos1:/]
user3=
@admins=rw

[/]
user3=r
user4=r

[/branches]
$authenticated = r
@admins = rw

[/tags]
* = 
@all = r
@admins = rw

[版本库1:/]
* = 
@管理组1 = rw

[版本库1:/项目1]
* = 
@组1 = rw
用户3 = r
        '''
        assert(self.f.tell()==0)
        self.f.write(buff)
        self.f.seek(0,0)
        self.authz.load(self.f)

    def tearDown(self):
        self.f.truncate(0)
    
    def testUser(self):
        user1 = User('Tom')
        self.assert_(unicode(user1) == 'Tom')
        self.assertRaises(Exception, User, "")
        user2 = User('Jerry')
        user3 = User('Zeb')
        self.assert_(user2<user1<user3)
        self.assert_('tom' in user1)
        self.assert_(user1 in user1)
        self.assert_(not 'Tom' in user2)
        alias1 = Alias('Tom')
        self.assert_(not alias1 in user1)
        alias1.user = user2
        self.assert_(alias1 in user2)
        self.assert_(not alias1 in user3)
        self.assert_(not None in user3)
        
    def testAlias(self):
        user1 = User('Tom')
        alias1=Alias('admin')
        alias1.user = user1
        self.assert_(unicode(alias1) == 'admin = Tom', unicode(alias1))
        alias2=Alias('manager', user1)
        self.assert_(unicode(alias2) == 'manager = Tom', unicode(alias1))

        self.assert_('&Manager' in alias2)
        self.assert_(not 'manager' in alias2)
        self.assert_('tom' in alias2)
        self.assert_(alias1 in alias2)
        self.assert_(user1 in alias2)
        self.assert_(not None in alias2)
        
        try:
            alias1.user=""
        except:
            pass
        else:
            fail("alias user point to User object, not others.")
        
        self.assertRaises(Exception, Alias, 'admin', 'Tom')
        self.assertRaises(Exception, Alias, '')
        self.assertRaises(Exception, Alias, '&admin')
        

    def testGroup(self):
        self.assertRaises(Exception, Group, '')
        self.assertRaises(Exception, Group, '@admin')
        user1 = User('user1')
        user2 = User('user2')
        user3 = User('user3')
        group2 = Group('team2')
        alias = Alias('admin')
        alias.user = user3
        self.assert_(group2.membernames == [])

        group1 = Group('team1')
        self.assert_(group1.name == 'team1')
        self.assert_(group1.uname == '@team1')
        group1.append([user1, user2])
        self.assert_(group1.membernames == ['user1','user2'], group1.membernames)

        group1.append([group2, alias])
        self.assert_(unicode(group1) == 'team1 = &admin, @team2, user1, user2', unicode(group1))
        self.assert_([m.uname for m in group1] == ['user1', 'user2', '@team2', '&admin'], [m.uname for m in group1])
        
        group1.remove('user1, &admin')
        self.assert_(group1.membernames == ['@team2', 'user2'], group1.membernames)
        group1.append(user1, alias)
        self.assert_(group1.membernames == ['&admin', '@team2', 'user1', 'user2'], group1.membernames)
        group1.remove([user2, '@team2', 'noexist'])
        self.assert_(group1.membernames == ['&admin', 'user1'], group1.membernames)
        group1.append(user2, group2)
        self.assert_(group1.membernames == ['&admin', '@team2', 'user1', 'user2'], group1.membernames)
        
        self.assertRaises(Exception, group2.append, group1, alias)
        group2.append(group1, alias, user1, autodrop=True)
        self.assert_([m.uname for m in group2] == ['&admin', 'user1'], [m.uname for m in group2])
        
        self.assert_(not 'admin' in group1)
        self.assert_('&admin' in group1)
        self.assert_(alias in group1)
        self.assert_('user3' in group1)
        self.assert_(user3 in group1)
        group1.append(Group('$authenticated'))
        self.assert_('nobody' in group1)
        group1.remove('$authenticated')
        self.assert_(not 'nobody' in group1)
        group1.append(Group('$anonymous'))
        self.assert_('*' in group1)
        self.assert_(not 'nobody' in group1)
        group1.remove('$anonymous')
        self.assert_(not '*' in group1)
        group1.append(Group('*'))
        self.assert_('*' in group1)
        self.assert_('nobody' in group1)
        self.assert_(not None in group1)
        self.assert_(not "  " in group1)
        
        group1.remove_all()
        self.assert_(group1.membernames == [], group1.membernames)
        
        g = Group('$authenticated')
        self.assert_(unicode(g) == "# Built-in group: $authenticated")

    def testIsValidName(self):
        self.assertRaises(Exception, check_valid_username, '蒋,鑫')
        self.assertRaises(Exception, check_valid_username, 'jiang;xin')
        self.assertRaises(Exception, check_valid_username, '')
        self.assertRaises(Exception, check_valid_username, User('user'))
        self.assertRaises(Exception, check_valid_reposname, 'my/repos')
        check_valid_reposname('/')
        
    def testUserList(self):
        ulist = UserList()
        self.assert_(list(ulist) == [])
        user1 = ulist.get_or_set('jiangxin')
        self.assert_(user1.name == 'jiangxin')
        self.assert_(user1.uname == 'jiangxin')
        self.assert_(list(ulist) == [user1])
        user2 = ulist.get_or_set('user2')
        self.assert_(user2.name == 'user2')
        self.assert_(user2.uname == 'user2')
        self.assert_(list(ulist) == [user1, user2])
        user3 = ulist.get('user3')
        self.assert_(user3==None)
        user3 = User('user3')
        self.assertRaises(Exception, ulist.get, user3)
        self.assert_([u.uname for u in ulist] == ['jiangxin', 'user2'], [u.uname for u in ulist])
        self.assert_(ulist.get_or_set('  ')==None)
        self.assertRaises(Exception, ulist.get_or_set, '&alias1')
        self.assertRaises(Exception, ulist.get_or_set, '@team1')
        self.assertRaises(Exception, ulist.get_or_set, '$anonymous')
        self.assertRaises(Exception, ulist.get_or_set, 'jiang,xin')

    def testAliasList(self):
        alist = AliasList()
        self.assert_(list(alist) == [])
        alias1 = alist.get_or_set('admin')
        self.assert_(alias1.name == 'admin')
        alias1 = alist.get('admin')
        self.assert_(alias1.uname == '&admin')
        self.assert_(list(alist) == [alias1])
        alias2 = alist.get_or_set('&root')
        self.assert_(alias2.name == 'root')
        self.assert_(alias2.uname == '&root')
        self.assert_(list(alist)== [alias1, alias2])
        self.assert_([a.uname for a in alist] == ['&admin', '&root'], [a.uname for a in alist])

        self.assert_(unicode(alist)=="[aliases]\nadmin = \nroot = \n", repr(unicode(alist)))
        
        self.assert_(alist.get_or_set('')==None)
        self.assert_(alist.get_or_set('  ')==None)
        self.assert_(alist.get('noexist')==None)
        self.assertRaises(Exception, alist.get_or_set, 'admin/user')
        
        assert alist.remove(alias1)==True
        self.assert_([a.uname for a in alist] == ['&root'], [a.uname for a in alist])
        alias1 = alist.get_or_set('admin')
        assert alist.remove('&admin')==True
        self.assert_([a.uname for a in alist] == ['&root'], [a.uname for a in alist])
        assert alist.remove('notexist')==False
        

    def testGroupList(self):
        user1 = User('user1')
        user2 = User('user2')
        alias = Alias('admin')
        alias.user=user1
        
        glist = GroupList()
        self.assert_(list(glist) == [])
        g1 = glist.get_or_set('team1')
        self.assert_(g1.name == 'team1')
        self.assert_(g1.uname == '@team1')
        self.assert_(list(glist) == [g1])
        g2 = glist.get_or_set('team2')
        self.assert_(g2.name == 'team2')
        self.assert_(g2.uname == '@team2')
        self.assert_(list(glist) == [g1, g2])

        g1.append(user1, g2)
        g2.append(user2, alias)

        self.assert_([g.uname for g in glist] == ['@team1', '@team2'], [g.uname for g in glist])

        self.assert_(unicode(glist)=="[groups]\nteam1 = @team2, user1\nteam2 = &admin, user2\n", repr(unicode(glist)))
        
        self.assert_(glist.get_or_set('')==None)
        self.assert_(glist.get_or_set('  ')==None)
        self.assert_(glist.get('noexist')==None)
        self.assertRaises(Exception, glist.get_or_set, 'admin,user')
        
        glist.remove(g1)
        self.assert_([g.uname for g in glist] == ['@team2'], [g.uname for g in glist])
        g1 = glist.get_or_set('team1')
        g1.append(user1, g2)
        self.assertRaises(Exception, glist.remove, '@team2')
        self.assertRaises(Exception, glist.remove, g2)
        self.assert_(unicode(glist)=='[groups]\nteam1 = @team2, user1\nteam2 = &admin, user2\n', repr(unicode(glist)))
        glist.remove('@team2',force=True)
        glist.remove('notexist')
        self.assert_(unicode(glist)=="[groups]\nteam1 = user1\n", repr(unicode(glist)))
        
    def testRules(self):
        user1 = User('user1')
        user2 = User('user2')
        user3 = User('user3')
        alias = Alias('admin')
        alias.user=user2
        group1 = Group('group1')
        group1.append(user1,alias)
        rule1 = Rule(group1)
        self.assert_(rule1.uname == '@group1', rule1.uname)
        self.assert_(rule1.rights == RIGHTS_NONE)
        rule1.rights = 'rw'
        self.assert_(rule1.rights == RIGHTS_ALL)
        rule1.rights = RIGHTS_ALL
        self.assert_(rule1.rights == RIGHTS_R|RIGHTS_W)
        self.assert_(unicode(rule1)=='@group1 = rw', repr(unicode(rule1)))
        self.assert_(rule1.get_permission(group1) == (RIGHTS_ALL, RIGHTS_NONE))
        self.assert_(rule1.get_permission(user1) == (RIGHTS_ALL, RIGHTS_NONE))
        self.assert_(rule1.get_permission(user2) == (RIGHTS_ALL, RIGHTS_NONE))
        self.assert_(rule1.get_permission('user2') == (RIGHTS_ALL, RIGHTS_NONE))
        self.assert_(rule1.get_permission(user3) == (RIGHTS_NONE, RIGHTS_NONE))
        
        group0 = Group('group0')
        rule0 = Rule(group0)
        rulelist = [rule1, rule0]
        rulelist.append('@choose...')
        self.assert_([unicode(r) for r in rulelist]==['@group1 = rw', '@group0 = ', '@choose...'], [unicode(r) for r in rulelist])
        self.assert_([unicode(r) for r in sorted(rulelist)]==['@group0 = ', '@group1 = rw', '@choose...'], [unicode(r) for r in sorted(rulelist)])
        

    def testModule(self):
        module = self.authz.get_module(u'版本库1', u'/项目1')
        self.assert_(module.fullname==u'版本库1:/项目1', module.fullname.encode('utf8'))
        self.assert_(unicode(module)==u'[版本库1:/项目1]\n* = \n@组1 = rw\n用户3 = r\n', unicode(module).encode('utf8'))

        module = Module('/', '/trunk')
        self.assert_(module.path == '/trunk')
        self.assert_(module.fullname == '/:/trunk')
        self.assert_(unicode(module)=='')
        
        obj = Group('* ')
        module.update_rule(obj,'')
        obj = Group('admins')
        module.update_rule(obj, 'rw')
        obj = Group('* ')
        module.update_rule(obj, 'r')
        obj = User(' jiang ')
        module.update_rule(obj, '')
        obj = Group(' $authenticated ')
        module.update_rule(obj, 'r')
        self.assert_(unicode(module) == 
'''[/trunk]
$authenticated = r
* = r
@admins = rw
jiang = 
''', repr(unicode(module)))
        self.assert_([unicode(r) for r in module]==['* = r', '@admins = rw', 'jiang = ', '$authenticated = r'], [unicode(r) for r in module])
        self.assert_([unicode(r) for r in module.rules]==['* = r', '@admins = rw', 'jiang = ', '$authenticated = r'], [unicode(r) for r in module.rules])
        
        module.del_rule('@admins= rw')
        self.assert_([unicode(r) for r in module]==['* = r', 'jiang = ', '$authenticated = r'], [unicode(r) for r in module])
        
        module = Module('myrepos', '')
        groupstar = Group('* ')
        module.update_rule(groupstar,'r')
        group1 = Group(' team1 ')
        module.update_rule(group1,'rw')
        groupstr = Group('*')
        module.update_rule(groupstr,'')
        user1 = User(' jiang ')
        module.update_rule(user1,'')
        groupauth = Group('$authenticated')
        module.update_rule(groupauth,'r')
        self.assert_(unicode(module) == '''[myrepos:/]
$authenticated = r
* = 
@team1 = rw
jiang = 
''', repr(unicode(module)))
        self.assert_(module.get_permit_bits(user1)==(RIGHTS_R, RIGHTS_ALL),module.get_permit_bits(user1))
        self.assert_(module.get_permit_str(user1)=='r',module.get_permit_str(user1))
        self.assert_(module.get_permit_str('@team1')=='rw',module.get_permit_str('@team1'))
        self.assert_(module.access_is_determined(user1))
        self.assert_(module.access_is_granted(user1,'r'))
        self.assert_(module.access_is_granted(user1,RIGHTS_R))
        self.assert_(module.access_is_granted(user1,'rw')==False)
        module.del_rule(('$authenticated = r','*='))
        self.assert_(module.access_is_determined(User("jiang")))
        self.assert_(module.access_is_granted(user1,''))
        self.assert_(module.access_is_determined(User("nobody"))==False)
        self.assertRaises(Exception, module.del_rule, None)
        self.assertRaises(Exception, module.del_rule, 'norepos', '/nopath', 'nobody=')
        
        module.clean_rules()
        self.assert_([unicode(r) for r in module]==[], [unicode(r) for r in module])

        self.assert_(Module('repos1', '/trunk')>Module('repos0', '/trunk'))
        self.assert_(Module('repos1', '/trunk')<"choice...")

    def testRepos(self):
        repos1 = self.authz.get_repos(u'版本库1')
        self.assert_(repos1.name==u'版本库1', repos1.name.encode('utf8'))

        user1=User('user1')
        user2=User('user2')
        user3=User('user3')
        alias1=Alias('alias1')
        alias1.user=user1
        group1=Group('group1')
        group2=Group('group2')
        
        repos1=Repos("myrepos1")
        self.assert_(repos1.name=="myrepos1")
        repos1.name="repos1"
        self.assert_(repos1.name=="repos1")
        self.assert_(repos1.is_blank()==True)

        self.assert_(repos1.admins=='')
        repos1.admins=[user1, alias1, group1]
        self.assert_(repos1.admins=='&alias1, @group1, user1', repos1.admins)
        self.assert_(repos1.is_blank()==False)
        repos1.del_admin([user1,])
        self.assert_(repos1.admins=='&alias1, @group1', repos1.admins)
        self.assertRaises(Exception, repos1.del_admin, '@group1')
        
        try:
            repos1.admins='user1'
        except:
            pass
        else:
            fail("repos admins set to user/group/alias object, not name.")
            
        mod1 = repos1.add_module('/trunk')
        self.assert_(mod1 == repos1.add_module('/trunk'))
        self.assert_(mod1 == repos1.get_module('/trunk'))
        self.assert_(None == repos1.get_module('/nothing'))
        mod2 = repos1.add_module('/tags')
        mod3 = repos1.add_module('/trunk/src')
        repos1.add_module('/wrong')
        mod4 = repos1.add_module('/branches')
        mod5 = repos1.del_module('/wrong')
        self.assert_([m.path for m in repos1]==['/trunk', '/tags', '/trunk/src', '/branches'], [m.path for m in repos1])
        self.assert_(repos1.path_list == ['/trunk', '/tags', '/trunk/src', '/branches'], repos1.path_list)

        mod1.update_rule(user1,'r')
        mod2.update_rule(alias1,'rw')

        self.assert_(unicode(repos1)=='[repos1:/tags]\n&alias1 = rw\n\n[repos1:/trunk]\nuser1 = r\n\n', repr(unicode(repos1)))
        
        repos1.del_all_modules()
        self.assert_(repos1.path_list == [], repos1.path_list)
        
        self.assert_(Repos('repos1') > Repos('repos0'))
        self.assert_(Repos('repos1') < 'astring')

    def testReposList(self):
        user1 = User('user1')
        user2 = User('user2')
        alias = Alias('admin')
        alias.user=user1
        
        rlist = ReposList()
        self.assert_([r.name for r in rlist] == ['/'], [r.name for r in rlist])
        r0 = rlist.get('/')

        r1 = rlist.get_or_set('repos1')
        self.assert_(r1.name == 'repos1')
        self.assert_(list(rlist) == [r0, r1])
        r2 = rlist.get_or_set('repos2')
        r2.add_module('/')
        module=r2.add_module('/trunk')
        module.update_rule(user1, 'r')
        self.assert_(r2.name == 'repos2')
        self.assert_(list(rlist) == [r0, r1, r2])

        self.assert_([r.name for r in rlist] == ['/', 'repos1', 'repos2'], [r.name for r in rlist])

        self.assert_(rlist.get('noexist')==None)
        self.assertRaises(Exception, rlist.get_or_set, 'wrong/repos')
        
        self.assert_(unicode(rlist)=='[repos2:/trunk]\nuser1 = r\n\n', repr(unicode(rlist)))
        
        self.assert_([r.name for r in rlist] == ['/', 'repos1', 'repos2'], [r.name for r in rlist])
        rlist.remove(r1)
        self.assert_([r.name for r in rlist] == ['/', 'repos2'], [r.name for r in rlist])
        rlist.remove(r2)
        self.assert_([r.name for r in rlist] == ['/', 'repos2'], [r.name for r in rlist])
        rlist.remove(r2, recursive=True)
        self.assert_([r.name for r in rlist] == ['/'], [r.name for r in rlist])
        self.assertRaises(Exception, rlist.remove, 'notexist')
        rlist.remove('/', recursive=False)
        rlist.remove('/', recursive=True)
        self.assert_([r.name for r in rlist] == ['/'], [r.name for r in rlist])
          
    def testReposAdmin(self):
        authz = SvnAuthz()
        u1=authz.add_user('u1')
        u2=authz.add_user('u2')
        u3=authz.add_user('u3')
        u4=authz.add_user('u4')
        u5=authz.add_user('u5')
        u6=authz.add_user('u6')
        u7=authz.add_user('u7')
        admin = authz.add_alias('admin', 'u1')
        team1 = authz.add_group('team1')
        self.assert_(unicode(team1)=='team1 = ', unicode(team1))
        team1 = authz.set_group('team1', None)
        self.assert_(unicode(team1)=='team1 = ', unicode(team1))
        team1 = authz.set_group('team1', 'u2, u3')
        self.assert_(unicode(team1)=='team1 = u2, u3', unicode(team1))
        team1 = authz.add_group_member('team1', u4)
        self.assert_(unicode(team1)=='team1 = u2, u3, u4', unicode(team1))
        repos1 = authz.add_repos('repos1')
        
        authz.set_admin('&admin, u7')
        authz.set_admin('@team1', 'repos1')
        self.assert_(authz.get_repos('/').admins == '&admin, u7')
        self.assert_(authz.get_repos('/repos1').admins == '@team1')
        authz.set_admin(', @team1, ', 'repos1')
        self.assert_(authz.get_repos('repos1').admins == '@team1')
        authz.set_admin(['&admin', 'u6'])
        self.assert_(authz.get_repos('/').admins == '&admin, u6')
        authz.set_admin([admin, u5])
        self.assert_(authz.get_repos('/').admins == '&admin, u5')

        authz.set_admin('')
        self.assert_(authz.get_repos('/').admins == '')
        authz.set_admin([])
        self.assert_(authz.get_repos('/').admins == '')
        authz.set_admin(None)
        self.assert_(authz.get_repos('/').admins == '')

    def testAuthzConfAcl(self):
        rl = self.authz.reposlist
        self.assert_(rl.get('/').name == '/')
        self.assert_(rl.get('/').admins == '&admin, root', rl.get('/').admins)
        self.assert_(rl.get('repos1').name == 'repos1')
        self.assert_(rl.get('repos1').admins == '@admin', rl.get('repos1').admins)
        self.assert_(rl.get('repos2').name == 'repos2', 'name: %s' % rl.get('repos2').name)
        self.assert_(rl.get('repos2').admins == 'admin2', rl.get('repos2').admins)
        self.assert_(self.authz.compose_acl() == 
u'''# admin : / = &admin, root
# admin : repos1 = @admin
# admin : repos2 = admin2
# admin : repos3 = admin3
# admin : 版本库1 = @管理组1
''', repr(self.authz.compose_acl()))
        pass

    def testAuthzConfAliases(self):
        al = self.authz.aliaslist
        self.assert_(al.get('admin').username == 'jiangxin', unicode(al.get('admin')))
        self.assert_(unicode(al) == u'[aliases]\n007 = james\nadmin = jiangxin\n别名1 = 用户1\n', repr(unicode(al)))
        pass

    def testAuthzConfGroups(self):
        gl = self.authz.grouplist
        self.assert_(sorted(gl.get('admins').membernames) == ['&007', '&admin'],
                     sorted(gl.get('admins').membernames))
        self.assert_(sorted(gl.get('team1').membernames) == 
                     ['@team2', 'user1', 'user11'],
                     sorted(gl.get('team1').membernames))
        self.assert_(sorted(gl.get('all').membernames) == 
                     ['@team1', 'user3', 'user4'],
                     sorted(gl.get('all').membernames))
        self.assert_(unicode(gl) == 
            u'''[groups]
admin = &admin, admin1, admin2, admin3
admins = &007, &admin
all = @team1, user3, user4
team1 = @team2, user1, user11
team2 = @team3, user2, user22
team3 = user3, user33
管理组1 = &别名1, 蒋鑫
组1 = &别名1, 用户2
''', repr(unicode(gl)))

    
    def testAuthzConfVersion(self):
        self.authz.load()
        self.assert_(self.authz.version == '0.0', self.authz.version)
        self.assert_(self.authz.get_revision_from_file() == '', self.authz.get_revision_from_file())
        
        buffs=[
               "",
               "#  version  : 0.1.100\n# admin:  /=admin\n[xxx]",
               "#version: 0.2.3a\n\n[groups]",
               ]
        result1=['',    "0.1.100", "0.2.3a"]
        result2=['0.0', "0.1.100", "0.2.3a"]
        result3=['0.1', "0.1.101", "0.2.0"]
        assert len(buffs)==len(result1)==len(result2)
        for i in range(0, len(buffs)):
            f=StringIO.StringIO(buffs[i])
            self.authz.load(f)
            self.assert_(self.authz.get_revision_from_file() == result1[i], self.authz.get_revision_from_file())
            self.assert_(self.authz.version == result2[i], self.authz.version)
            lastv = self.authz.version
            self.authz.save(self.authz.version)
            self.assert_(self.authz.version == result3[i], self.authz.version)
            self.assertRaises(Exception, self.authz.save, lastv)


    def testAuthzConfRepos(self):
        # blank configuration
        self.authz.load()
        # add_repos
        self.assert_(isinstance(self.authz.add_repos('repos1'), Repos))
        self.assert_(isinstance(self.authz.add_repos('repos2'), Repos))
        self.assert_(','.join(map(lambda x:x.name, self.authz.reposlist)) ==
                     '/,repos1,repos2', ','.join(map(lambda x:x.name,
                                                     self.authz.reposlist)))
        # add_admin
        self.assert_(self.authz.set_admin('admin1,admin2') == True)
        self.assert_(self.authz.is_admin('admin1','/') == True)
        self.assert_(self.authz.set_admin('adminx', 'repos1') == True)
        self.assert_(self.authz.is_admin('adminx', 'repos1') == True)
        self.assert_(self.authz.is_super_user('admin1') == True)
        self.assert_(self.authz.is_super_user('adminx') == False)

        self.assert_(self.authz.get_manageable_repos_list('admin1') == ['/', 'repos1', 'repos2'], self.authz.get_manageable_repos_list('admin1'))
        self.assert_(self.authz.get_manageable_repos_list('adminx') == ['repos1'], self.authz.get_manageable_repos_list('adminx'))
        self.assert_(self.authz.get_manageable_repos_list('adminxyz') == [])
        self.assert_(self.authz.get_manageable_repos_list('') == [])
        # add_module (repos = /)
        self.assert_(isinstance(self.authz.add_module('/', '/trunk///'), Module))
        m = self.authz.get_module('/', '/trunk/')
        self.assert_(isinstance(m, Module))
        self.assert_(m.path == '/trunk')
        self.assert_(m.repos == '/')
        # add_module (path = /)
        self.assert_(isinstance(self.authz.add_module('repos1', '/'), Module))
        m = self.authz.get_module('repos1', '')
        self.assert_(m.repos+':'+m.path == 'repos1:/')
        self.assert_(','.join(map(lambda x:x.repos+':'+x.path,
                                  self.authz.modulelist())) ==
                     '/:/trunk,repos1:/', ','.join(map(lambda
                                                       x:x.repos+':'+x.path,
                                                       self.authz.modulelist())))

        # add_alias
        user = self.authz.add_user('jiangxin')
        self.assert_(isinstance(self.authz.add_alias('superuser', user), Alias))
        self.assert_(isinstance(self.authz.add_alias('root', user), Alias))
        self.assert_(unicode(self.authz.aliaslist) == '[aliases]\nroot = jiangxin\nsuperuser = jiangxin\n', repr(unicode(self.authz.aliaslist)))
        self.assert_(','.join(map(lambda x:x.name, self.authz.aliaslist)) ==
                     'root,superuser', ','.join(map(lambda x:x.name,
                                                    self.authz.aliaslist)))

        # add_group
        self.assert_(unicode(self.authz.grouplist) == '[groups]\n', repr(unicode(self.authz.grouplist)))
        self.assert_(isinstance(self.authz.add_group('myteam','user1'), Group))
        self.assert_(unicode(self.authz.grouplist) == '[groups]\nmyteam = user1\n', repr(unicode(self.authz.grouplist)))
        self.assert_(isinstance(self.authz.add_group_member('myteam','user2,user3'), Group))
        self.assert_(isinstance(self.authz.add_group_member('myteam','user2,user3'), Group))
        self.assert_(unicode(self.authz.grouplist) == '[groups]\nmyteam = user1, user2, user3\n', repr(unicode(self.authz.grouplist)))
        self.assert_(','.join(map(lambda x:x.name, self.authz.grouplist)) ==
                     'myteam', ','.join(map(lambda x:x.name,
                                            self.authz.grouplist)))
        self.assert_(isinstance(self.authz.add_group_member('team1','@team2'),Group))
        self.assert_(isinstance(self.authz.add_group_member('myteam','@team1, @team2, *, $authenticated'),Group))
        self.assert_(isinstance(self.authz.add_group_member('team2','$authenticated,*'),Group))
        self.assert_(isinstance(self.authz.add_group_member('team3','@team4'),Group))
        self.assert_(isinstance(self.authz.add_group_member('team4','@myteam'),Group))
        self.assertRaises(Exception, self.authz.add_group_member, 'myteam','@team3, @team4, @team5')
        self.assert_(unicode(self.authz.grouplist.get('@myteam')) == 
                     'myteam = $authenticated, *, @team1, @team2, user1, user2, user3', 
                     repr(unicode(self.authz.grouplist.get('@myteam')))) 
        self.assert_(isinstance(self.authz.add_group_member('myteam','@team3, @team4, @team5', True),Group))
        self.assert_(unicode(self.authz.grouplist.get('@myteam')) == 
                     'myteam = $authenticated, *, @team1, @team2, @team5, user1, user2, user3', 
                     repr(unicode(self.authz.grouplist.get('@myteam')))) 
        self.assertRaises(Exception, self.authz.del_group, '@team2')
        self.assert_(unicode(self.authz.grouplist.get('@team2')) == 'team2 = $authenticated, *', 
                     self.authz.grouplist.get('@team2'))
        self.authz.del_group('@team2',force=True)
        self.assert_(self.authz.grouplist.get('@team2') == None, 
                     unicode(self.authz.grouplist.get('@team2')))
        self.assert_(unicode(self.authz.grouplist.get('@myteam')) == 
                     'myteam = $authenticated, *, @team1, @team5, user1, user2, user3',
                     repr(unicode(self.authz.grouplist.get('@myteam')))) 
        self.assert_(unicode(self.authz.grouplist.get('@team1')) == 'team1 = ',
                     repr(unicode(self.authz.grouplist.get('@team1')))) 

        # add_rule 
        module = self.authz.get_module('/', '/trunk/')
        tmpstr = "%s" % module
        self.assert_(tmpstr =='', tmpstr)
        self.assertRaises(Exception, self.authz.set_rules, 'norepos', '/nopath', '*=')
        self.authz.set_rules('/', '/trunk', 'user1=r\nuser1 = rw\n user2 =\n')
        tmpstr = "%s" % module
        self.assert_(tmpstr ==u'[/trunk]\nuser1 = rw\nuser2 = \n', repr(tmpstr))
        self.authz.set_rules('/', '/trunk', '')
        tmpstr = "%s" % module
        self.assert_(tmpstr ==u'', repr(tmpstr))

        self.assertRaises(Exception, self.authz.add_rules, 'norepos', '/nopath', '*=')
        self.assertRaises(Exception, self.authz.add_rules, '/', '/trunk', '*')
        self.assertRaises(Exception, self.authz.add_rules, '/', '/trunk', ['*'])
        self.assertRaises(Exception, self.authz.add_rules, '/', '/trunk', None)
        self.assert_(self.authz.add_rules('/', '/trunk/', '&superuser=r') == True, self.authz.add_rules('/', '/trunk/', '&superuser=rw'))
        self.assert_(self.authz.add_rules('repos1', '/', '*=') == True, self.authz.add_rules('repos1', '/', '*=r'))
        self.assert_(self.authz.add_rules('/', '/trunk/', '&superuser=r; *=rw') == True, self.authz.add_rules('/', '/trunk/', '&superuser=rw'))
        self.assert_(self.authz.add_rules('/', '/trunk/', '&superuser=rw\n *=r') == True, self.authz.add_rules('/', '/trunk/', '&superuser=rw'))
        self.assert_(unicode(self.authz.get_module('repos1', '/')) == '[repos1:/]\n* = r\n', repr(unicode(self.authz.get_module('repos1', '/'))))
        self.assert_(self.authz.add_rules('repos1', '/', ['*=', '@team1=rw']) == True)
        self.assert_(unicode(self.authz.get_module('repos1', '/')) == '[repos1:/]\n* = \n@team1 = rw\n', repr(unicode(self.authz.get_module('repos1', '/'))))
        self.assert_(','.join(map(lambda x:unicode(x), self.authz.rulelist())) ==
                     '&superuser = rw,* = r,* = ,@team1 = rw', ','.join(map(lambda
                                                                      x:unicode(x),
                                                                      self.authz.rulelist())))
        self.assertRaises(Exception, self.authz.chk_grp_ref_by_rules, '*')
        self.assertRaises(Exception, self.authz.chk_grp_ref_by_rules, '@team1')
        self.authz.chk_grp_ref_by_rules('@team2')

        self.assert_(self.authz.del_rule('norepos', '/nopath', ['*=rw']) == False)
        self.assert_(self.authz.del_rule('repos1', '/', ['*=rw']) == True)
        self.assert_(unicode(self.authz.get_module('repos1', '/')) == '[repos1:/]\n@team1 = rw\n', repr(unicode(self.authz.get_module('repos1', '/'))))
        self.assert_(','.join(map(lambda x:x.name, self.authz.grouplist)) ==
                     '$authenticated,*,myteam,team1,team3,team4,team5',
                     ','.join(map(lambda x:x.name, self.authz.grouplist)))

        # del_alias
        self.authz.chk_alias_ref_by_rules('&root')
        self.authz.chk_alias_ref_by_rules('root')
        self.authz.del_alias('&root')
        self.assert_(self.authz.del_alias('notexist')==False)
        self.assertRaises(Exception, self.authz.chk_alias_ref_by_rules, '&superuser')
        self.assertRaises(Exception, self.authz.del_alias, '&superuser')
        self.assert_(','.join(map(lambda x:x.name, self.authz.aliaslist)) ==
                     'superuser', ','.join(map(lambda x:x.name,
                                               self.authz.aliaslist)))

        # del_group
        self.assertRaises(Exception, self.authz.chk_grp_ref_by_rules,'@team1')
        self.assertRaises(Exception, self.authz.del_group,'@team1')
        self.authz.chk_grp_ref_by_rules('myteam')
        self.assertRaises(Exception, self.authz.del_group,'@myteam')
        self.assert_(','.join(map(lambda x:x.name, self.authz.grouplist)) ==
                     '$authenticated,*,myteam,team1,team3,team4,team5',
                     ','.join(map(lambda x:x.name, self.authz.grouplist)))
        self.assert_(self.authz.del_group('@myteam',force=True) == True)
        self.assert_(','.join(map(lambda x:x.name, self.authz.grouplist)) ==
                     '$authenticated,*,team1,team3,team4,team5',
                     ','.join(map(lambda x:x.name, self.authz.grouplist)))

        # remove
        self.assert_(isinstance(self.authz.add_group_member('myteam','user1,user2,user3'),Group))
        self.assert_(self.authz.del_group_member('myteam','user1,user3') == True)
        self.assert_(unicode(self.authz.grouplist) == 
                     '[groups]\nmyteam = user2\nteam1 = \nteam3 = @team4\nteam4 = \nteam5 = \n',
                     repr(unicode(self.authz.grouplist)))

        # del_module
        self.assert_(self.authz.del_module('norepos', '/nopath') == False)
        self.assert_(self.authz.del_module('repos2', '/') == False)
        self.assert_(self.authz.del_module('repos2', '/noexist') == False)
        self.assert_(self.authz.del_module('repos1', '/') == True)
        self.assert_(self.authz.get_module('repos1', '/') == None)

        # del_repos
        #self.assert_(unicode(self.authz) == '', unicode(self.authz))
        self.assert_(self.authz.get_repos('/').is_blank() == False)
        self.assert_(self.authz.get_repos('repos2').is_blank() == True)
        self.assert_(self.authz.get_repos('repos1').is_blank() == False)
        self.assert_(self.authz.del_repos('/') == False)
        self.assert_(self.authz.del_repos('/', recursive=True) == True)
        self.assert_(self.authz.get_repos('/').is_blank() == False)
        self.assert_(self.authz.del_repos('repos2') == True)
        self.assert_(self.authz.del_repos('repos1') == False)
        self.assert_(self.authz.del_repos('repos1',recursive=True) == True)

        # output config from __str__
        self.assert_(unicode(self.authz) == 
                     '# version : 0.0\n# admin : / = admin1, admin2\n\n[groups]\nmyteam = user2\nteam1 = \nteam3 = @team4\nteam4 = \nteam5 = \n\n[aliases]\nsuperuser = jiangxin\n\n', 
                     repr(unicode(self.authz)))

    def testAuthzConfDefault(self):
        # test save config
        self.assert_(self.authz.version=='0.1.1', self.authz.version)
        self.authz.save(self.authz.version)
        self.assert_(self.authz.version=='0.1.2', self.authz.version)
        
        self.assertRaises(Exception, self.authz.del_alias, "007", force=True)
        self.assertRaises(Exception, self.authz.del_alias, "admin", force=False)
        
        # is_admin()
        self.assert_(self.authz.get_userobj('noexistuser')==None)
        self.assert_(self.authz.get_userobj('')==None)
        self.assert_(self.authz.is_admin('jiangxin') == True)
        self.assert_(self.authz.is_admin(User('jiangxin')) == True)
        self.assert_(self.authz.is_admin(self.authz.get_userobj('&admin')) == True)
        self.assert_(self.authz.is_admin('') == False)
        self.assert_(self.authz.is_admin('root') == True, self.authz.is_admin('root'))
        self.assert_(self.authz.is_super_user('jiangxin') == True)
        self.assert_(self.authz.is_admin('&admin') == True)
        self.assert_(self.authz.is_admin('admin1') == False)
        self.assert_(self.authz.is_admin('admin1','repos1') == True)
        self.assert_(self.authz.is_admin('admin4','repos1') == False)
        self.assert_(self.authz.is_admin('admin1','repos2') == False)
        self.assert_(self.authz.is_admin('admin2','repos2') == True)
        self.assert_(self.authz.is_admin('admin10','repos2') == False)
        self.assert_(self.authz.is_admin('','repos3') == False)
        self.assert_(self.authz.is_admin('jiangxin','repos123') == True)

        # add_admin() test
        self.authz.set_admin('admin1,admin2')
        self.authz.set_admin(['admin1','admin2'],'repos1')
        self.assertRaises(Exception, self.authz.set_admin, 1,'repos1')
        # reposx does not exist.
        self.authz.set_admin('admin2','reposx')
        self.assert_(self.authz.is_admin('admin1','repos2') == True)


        # SvnAuthz __str__ test
        self.authz.update_revision()
        self.assert_(unicode(self.authz) ==
u"""# version : 0.1.3
# admin : / = admin1, admin2
# admin : repos1 = admin1, admin2
# admin : repos2 = admin2
# admin : repos3 = admin3
# admin : 版本库1 = @管理组1

[groups]
admin = &admin, admin1, admin2, admin3
admins = &007, &admin
all = @team1, user3, user4
team1 = @team2, user1, user11
team2 = @team3, user2, user22
team3 = user3, user33
管理组1 = &别名1, 蒋鑫
组1 = &别名1, 用户2

[aliases]
007 = james
admin = jiangxin
别名1 = 用户1

[/]
user3 = r
user4 = r

[/branches]
$authenticated = r
@admins = rw

[/tags]
* = 
@admins = rw
@all = r

[/trunk]\nuser2 = 

[/trunk/src]
user1 = r
user2 = r

[repos1:/]
@admins = rw
user3 = 

[repos1:/trunk]
user1 = r
user2 = 

[repos1:/trunk/src]
&007 = r
user1 = 

[版本库1:/]
* = 
@管理组1 = rw

[版本库1:/项目1]
* = 
@组1 = rw
用户3 = r

""", (repr(unicode(self.authz))))

        self.assert_(self.authz.check_rights('*','/','/trunk/src/test','r') == False)
        self.assert_(self.authz.check_rights(None,None,'/trunk/src/test','r') == False)
        self.assert_(self.authz.check_rights('user1','repos1','/trunk/src/test','r') == False)
        self.assert_(self.authz.check_rights('user1','repos1','/trunk/src/test',RIGHTS_R) == False)
        self.assert_(self.authz.check_rights('user1','GLOBAL','/trunk/src/test','r') == True)
        self.assert_(self.authz.check_rights('user2','repos1','/trunk/src/test','r') == True)
        self.assert_(self.authz.check_rights('user2','repos1','/trunk','r') == False)
        self.assert_(self.authz.check_rights('user2','GLOBAL','/trunk','r') == False)
        self.assert_(self.authz.check_rights('user3','repos1','/trunk','r') == False)
        self.assert_(self.authz.check_rights('user4','repos1','/trunk','r') == True)
        self.assert_(self.authz.check_rights('user4','GLOBAL','/trunk','r') == True)
        self.assert_(self.authz.check_rights('user5','GLOBAL','/trunk','r') == False, self.authz.check_rights('user5','GLOBAL','/trunk','r'))

        self.assert_(self.authz.check_rights('user5','GLOBAL','/branches/a/b/c','r') == True)

        self.assert_(self.authz.get_access_map("", "reposnoexist", descend=False)==None,
                     self.authz.get_access_map("", "reposnoexist", descend=False))
         
        self.authz.add_rules('/','/trunk/src', 'jiangxin=r')
        self.assert_(self.authz.get_access_map_msgs('jiangxin', 'reposnoexist') == 
["""
==================================================
Access map on 'reposnoexist' for user 'jiangxin'
==================================================
  * Writable:
    /branches
    /tags
----------------------------------------
  * Readable:
    /trunk/src
----------------------------------------
  * Denied:
    /
    /trunk
----------------------------------------
"""], repr(self.authz.get_access_map_msgs('jiangxin', 'reposnoexist')))

        self.assert_(self.authz.get_access_map_msgs('jiangxin', 'reposnoexist', abbr=True) == 
[u'''
jiangxin => [reposnoexist]
----------------------------------------
RW: /branches, /tags
RO: /trunk/src
XX: /, /trunk

'''], repr(self.authz.get_access_map_msgs('jiangxin', 'reposnoexist', abbr=True)))

        self.assert_(self.authz.get_path_access_msgs('jiangxin', 'noexist', '/branches/1.0') == ['User jiangxin has Full (RW) rights for module noexist:/branches/1.0'],
                     self.authz.get_path_access_msgs('jiangxin', 'noexist', '/branches/1.0'))
        self.assert_(self.authz.get_path_access_msgs('jiangxin', 'noexist', '/branches/1.0', abbr=True) == ['[noexist:/branches/1.0] jiangxin = rw'],
                     self.authz.get_path_access_msgs('jiangxin', 'noexist', '/branches/1.0', abbr=True))

        self.assert_(self.authz.get_path_access_msgs('jiangxin', 'noexist', '/trunk/src') == ['User jiangxin has ReadOnly (RO) rights for module noexist:/trunk/src'],
                     self.authz.get_path_access_msgs('jiangxin', 'noexist', '/trunk/src'))
        self.assert_(self.authz.get_path_access_msgs('jiangxin', 'noexist', '/trunk/src', abbr=True) == ['[noexist:/trunk/src] jiangxin = r'],
                     self.authz.get_path_access_msgs('jiangxin', 'noexist', '/trunk/src', abbr=True))

        self.assert_(self.authz.get_path_access_msgs('jiangxin', 'noexist', '/new') == ['User jiangxin can *NOT* access to module noexist:/new'],
                     self.authz.get_path_access_msgs('jiangxin', 'noexist', '/new'))
        self.assert_(self.authz.get_path_access_msgs('jiangxin', 'noexist', '/new', abbr=True) == ['[noexist:/new] jiangxin ='],
                     self.authz.get_path_access_msgs('jiangxin', 'noexist', '/new', abbr=True))

        self.assert_(self.authz.get_access_map_msgs('  ', abbr=True) == 
[u"""
* => [/]
----------------------------------------
RW: 
RO: 
XX: /, /branches, /tags, /trunk, /trunk/src

""", 
u"""
* => [repos1]
----------------------------------------
RW: 
RO: 
XX: /, /branches, /tags, /trunk, /trunk/src

""",
u"""
* => [repos2]
----------------------------------------
RW: 
RO: 
XX: /, /branches, /tags, /trunk, /trunk/src

""",
u"""
* => [repos3]
----------------------------------------
RW: 
RO: 
XX: /, /branches, /tags, /trunk, /trunk/src

""",
u"""
* => [版本库1]
----------------------------------------
RW: 
RO: 
XX: /, /branches, /tags, /trunk, /trunk/src, /项目1

"""]
, repr(self.authz.get_access_map_msgs('  ', abbr=True)))

        self.assert_(self.authz.get_path_access_msgs(' ', '*', '/trunk/src/mod1', abbr=True) == 
                     [u'[/:/trunk/src/mod1] * =', 
                      u'[repos1:/trunk/src/mod1] * =', 
                      u'[repos2:/trunk/src/mod1] * =', 
                      u'[repos3:/trunk/src/mod1] * =',
                      u'[版本库1:/trunk/src/mod1] * =',],
                     self.authz.get_path_access_msgs(' ', '*', '/trunk/src/mod1', abbr=True))

        gadmin = self.authz.get_userobj('@admin')
        gadmins= self.authz.get_userobj('@admins')
        self.assert_('&admin' in gadmin)
        self.assert_('&admin' in gadmins)
        self.authz.del_alias("admin", force=True)
        self.assert_(not '&admin' in gadmin)
        self.assert_(not '&admin' in gadmins)
        self.authz.add_alias('008', 'zhouxx')
        self.authz.del_alias('008')
        self.authz.add_alias('008', 'zhouxx')
        self.authz.add_rules('repos1', '/trunk/src', '&008=')
        self.assertRaises(Exception, self.authz.del_alias, '008')

if __name__ == '__main__': 
    import unittest
    unittest.main()




# -*- coding: utf-8 -*-
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

"""The base Controller API

Provides the BaseController class for subclassing.
"""
from pylons.controllers import WSGIController
from pylons.templating import render_mako as render

import pysvnmanager.lib.helpers as h
from pylons.controllers.util import abort, etag_cache
from pylons import app_globals, cache, config, request, response, session
from pylons.i18n import _, ungettext, N_
from pylons.i18n import set_lang, add_fallback
from pylons import app_globals as g
from pylons import tmpl_context as c
from pylons import url
from pylons.controllers.util import redirect

import sys
config_path = config["here"] + '/config'
if config_path not in sys.path:
    sys.path.insert(0, config_path)
from localconfig import LocalConfig as cfg

class BaseController(WSGIController):
    requires_auth = []

    def __before__(self, action=None):
        if 'lang' in session:
            set_lang(session['lang'])
        #log.debug(request.languages)
        try:
            for lang in request.languages:
                if lang.lower() in ['zh-cn', 'zh']:
                    add_fallback('zh')
                elif lang in ['en']:
                    add_fallback(lang)
        except:
            pass

        ## Show exception and traceback info
        if getattr(g, 'catch_e', None):
            return redirect(url(controller='template', action='show_e'))
        else:
           g.catch_e = []

        if isinstance(self.requires_auth, bool) and not self.requires_auth:
            pass
        elif isinstance(self.requires_auth, (list, tuple)) and \
            not action in self.requires_auth:
            pass
        else:
            if 'user' not in session:
                session['path_before_login'] = request.path_info
                session.save()
                return redirect(url('login'))

    def __call__(self, environ, start_response):
        """Invoke the Controller"""
        # WSGIController.__call__ dispatches to the Controller method
        # the request is routed to. This routing information is
        # available in environ['pylons.routes_dict']
        return WSGIController.__call__(self, environ, start_response)

# Include the '_' function in the public names
#__all__ = [__name for __name in locals().keys() if not __name.startswith('_') \
#           or __name == '_']



# -*- coding: utf-8 -*-
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.


import locale

def to_unicode(text, charset=None, escape=False):
    """Convert a `str` object to an `unicode` object.
    """
    utext = u''
    if not isinstance(text, str):
        if isinstance(text, Exception):
            # two possibilities for storing unicode strings in exception data:
            try:
                # custom __str__ method on the exception (e.g. PermissionError)
                utext = unicode(text)
            except UnicodeError:
                # unicode arguments given to the exception (e.g. parse_date)
                utext = ' '.join([to_unicode(arg, charset, False) for arg in text.args])
        else:
            utext = unicode(text)
    else:
        if charset:
            utext = unicode(text, charset, 'replace')
        else:
            try:
                utext = unicode(text, 'utf-8')
            except UnicodeError:
                utext = unicode(text, locale.getpreferredencoding(), 'replace')
    if escape:
        return utext.encode('raw_unicode_escape')
    else:
        return utext

def to_utf8(text, charset='gb18030', escape=False):
    """Convert a string to UTF-8, assuming the encoding is either UTF-8, ISO
    Latin-1, or as specified by the optional `charset` parameter.
    """
    utext = ''
    if isinstance(text, unicode):
        utext = text.encode('utf-8')
    elif not isinstance(text, str):
        utext = unicode(text).encode('utf-8')
    else:
        try:
            # Do nothing if it's already utf-8
            u = unicode(text, 'utf-8')
            utext = text
        except UnicodeError:
            try:
                # Use the user supplied charset if possible
                u = unicode(text, charset)
            except UnicodeError:
                # This should always work
                u = unicode(text, 'iso-8859-15')
            utext = u.encode('utf-8')
    if escape:
        return repr(utext)[1:-1]
    else:
        return utext




# -*- coding: utf-8 -*-
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

"""Pylons application test package

This package assumes the Pylons environment is already loaded, such as
when this script is imported from the `nosetests --with-pylons=test.ini`
command.

This module initializes the application via ``websetup`` (`paster
setup-app`) and provides the base testing objects.
"""
from unittest import TestCase

from paste.deploy import loadapp
from paste.script.appinstall import SetupCommand
from pylons import url
from routes.util import URLGenerator
from webtest import TestApp

import pylons.test

__all__ = ['environ', 'url', 'TestController']

# Invoke websetup with the current config file
SetupCommand('setup-app').run([pylons.test.pylonsapp.config['__file__']])

environ = {}

import os
import sys
from shutil import copyfile

class TestController(TestCase):

    def __init__(self, *args, **kwargs):
        self.authz_file = os.path.dirname(__file__) + '/../../config/svn.access.test'

        wsgiapp = pylons.test.pylonsapp
        config = wsgiapp.config
        self.app = TestApp(wsgiapp)
        url._push_object(URLGenerator(config['routes.map'], environ))
        TestCase.__init__(self, *args, **kwargs)

    def rollback(self):
        src = os.path.dirname(__file__) + '/../config/svn.access.example'
        dest = self.authz_file
        copyfile(src, dest)
        
    def load_authz(self):
        from pysvnmanager.model import svnauthz
        return svnauthz.SvnAuthz(self.authz_file)
        
    def login(self, username, password=""):
       
        if not password:
            wsgiapp = pylons.test.pylonsapp
            config = wsgiapp.config
            d = eval(config.get('test_users', {}))
            password = d.get(username,'')

        r = self.app.post(url(controller='security', action='submit'), params={'username': username, 'password': password})

        #res = self.app.get(url(controller='security', action='index'))
        #form = res.forms[0]
        #form['username'] = username
        #if not password:
        #    wsgiapp = pylons.test.pylonsapp
        #    config = wsgiapp.config
        #    d = eval(config.get('test_users', {}))
        #    password = d.get(username,'')
        #form['password'] = password
        #form.submit()



# -*- coding: utf-8 -*-
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

"""Helper functions

Consists of functions to typically be used within templates, but also
available to Controllers. This module is available to templates as 'h'.
"""
# Import helpers as desired, or define your own, ie:
#from webhelpers.html.tags import checkbox, password

from webhelpers.html import escape, HTML, literal, url_escape
from webhelpers.html.tags import *
from pylons import url
from pylons.controllers.util import redirect



# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import logging

from pysvnmanager.lib.base import *
from pylons.i18n import _, ungettext, N_

log = logging.getLogger(__name__)

class SecurityController(BaseController):

    def index(self):
        """
        Show login form. Submits to login/submit
        """
        return render('/login/login.mako')

    def submit(self):
        """
        Verify username and password
        """
        auth_passed = False
        # Both fields filled?
        username = request.params.get('username')
        password = request.params.get('password')

        for auth in cfg.auth:
            if auth(username=username, password=password, config=cfg):
                auth_passed = True
                break
        
        # Mark user as logged in
        if auth_passed:
            session['user'] = username
            log.info(_(u"User %s logged in") % session['user'])
            session.save()

            # Send user back to the page he originally wanted to get to
            if session.get('path_before_login'):
                redirect(url(session['path_before_login']))
            else: # if previous target is unknown just send the user to a welcome page
                redirect(url(controller='check',action='index'))
        else:
            log.error("pySvnManager: User %s login failed from host [%s]" % ( username, request.remote_addr))
            session.clear()
            session.save()
            c.login_message = _(u"Login failed for user: %s") % username
            return render('/login/login.mako')

    def logout(self):
        """
        Logout the user and display a confirmation message
        """
        if 'user' in session:
            log.info(_("User %s logged out") % session['user'])
            del session['user']
            session.save()
        redirect(url("login"))

    def failed(self):
        return render('/auth_failed.mako')



"""Filename matching with shell patterns.

fnmatch(FILENAME, PATTERN) matches ignore case.
fnmatchcase(FILENAME, PATTERN) always takes case in account.

The functions operate by translating the pattern into a regular
expression.  They cache the compiled regular expressions for speed.

The function translate(PATTERN) returns a regular expression
corresponding to PATTERN.  (It does not compile it.)

Double or multiple stars has special meaning: include any characters
including the path seperater '/', while single star matches characters
except '/'.  -- Hacked by Jiang Xin. <jiangxin AT ossxp.com>
"""

import re

__all__ = ["filter", "fnmatch","fnmatchcase","translate"]

_cache = {}

def fnmatch(name, pat):
    """Test whether FILENAME matches PATTERN.

    Patterns are Unix shell style:

    *       matches everything except '/'
    **      matches everything including '/'
    ?       matches any single character
    [seq]   matches any character in seq
    [!seq]  matches any char not in seq

    An initial period in FILENAME is not special.
    Both FILENAME and PATTERN are case-insensitive.
    If you don't want this, use fnmatchcase(FILENAME, PATTERN).
    """

    import os
    name = name.lower()
    pat = pat.lower()
    return fnmatchcase(name, pat)

def filter(names, pat):
    """Return the subset of the list NAMES that match PAT"""
    import os,posixpath
    result=[]
    pat=os.path.normcase(pat)
    if not pat in _cache:
        res = translate(pat)
        _cache[pat] = re.compile(res)
    match=_cache[pat].match
    if os.path is posixpath:
        # normcase on posix is NOP. Optimize it away from the loop.
        for name in names:
            if match(name):
                result.append(name)
    else:
        for name in names:
            if match(os.path.normcase(name)):
                result.append(name)
    return result

def fnmatchcase(name, pat):
    """Test whether FILENAME matches PATTERN, including case.

    This is a version of fnmatch() which doesn't case-normalize
    its arguments.
    """

    if not pat in _cache:
        res = translate(pat)
        _cache[pat] = re.compile(res)
    return _cache[pat].match(name) is not None

def translate(pat):
    """Translate a shell PATTERN to a regular expression.

    There is no way to quote meta-characters.
    """

    i, n = 0, len(pat)
    multistars = False
    res = ''
    while i < n:
        c = pat[i]
        i = i+1
        if c == '*':
            while i < n:
                c = pat[i]
                if c != '*': 
                    break
                i = i+1
                multistars = True
            if multistars:
                res = res + '.*'
            else:
                res = res + '[^/]*'
        elif c == '?':
            res = res + '.'
        elif c == '[':
            j = i
            if j < n and pat[j] == '!':
                j = j+1
            if j < n and pat[j] == ']':
                j = j+1
            while j < n and pat[j] != ']':
                j = j+1
            if j >= n:
                res = res + '\\['
            else:
                stuff = pat[i:j].replace('\\','\\\\')
                i = j+1
                if stuff[0] == '!':
                    stuff = '^' + stuff[1:]
                elif stuff[0] == '^':
                    stuff = '\\' + stuff
                res = '%s[%s]' % (res, stuff)
        else:
            res = res + re.escape(c)
    return res + "$"



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, getopt, os, popen2
import pickle
from StringIO import StringIO

# Python 2.3 is required by PyRSS2Gen
py_version  = sys.version_info
if sys.version_info[0:2] < (2,3):
    sys.stderr.write("Error: Python 2.3 or higher required")
    sys.exit(1)
    
import datetime

try:
    import PyRSS2Gen
except ImportError:
    sys.stderr.write("Please install PyRSS2Gen before running this script\n")
    sys.stderr.write("PyRSS2Gen can be downloaded from: \n")
    sys.stderr.write("http://www.dalkescientific.com/Python/PyRSS2Gen.html\n")

def usage():
    print "Usage: svn2rss.py [-h|--help] [--svn-path] --revision <rev> "
    print "                  --repos-path <path> "
    print "                  --url <url> --rss-file <file>"
    print "       svn-path : path where svn binaries are installed"
    print "       url      : link in the rss item that points to the"
    print "                  viewcvs page for the revision"
    print ""
    print "Generates a RSS 2.0 file containing commit information."
    print "Once the maximum number of items is reached, the oldest element"
    print "is removed.  The item title is the Revision number and the item"
    print "description contains the author, date, log messages and changed"
    print "paths."
    

if len(sys.argv) == 1:
    usage()
    sys.exit(2)
   
try:
    opts, args = getopt.getopt(sys.argv[1:],"h", ["help", "svn-path=",
                                                    "revision=",
                                                    "repos-path=", "url=",
                                                    "rss-file="])
except getopt.GetoptError, msg:
    print msg
    sys.stderr.write(usage())
    sys.exit(2)

for opt, arg in opts:
    if opt in ("-h", "--help"):
        usage()
        sys.exit(0)
    elif opt == "--svn-path":
        svn_path = arg
    elif opt == "--revision":
        commit_rev = arg
    elif opt == "--repos-path":
        repos_path = arg
    elif opt == "--url":
        url = arg
    elif opt == "--rss-file":
        rss_file = arg

class SVN2RSS:
    def __init__(self, svn_path, revision, repos_path, url, rss_file):
        self.max_items = 20
        self.svn_path = svn_path
        self.revision = revision
        self.repos_path = repos_path
        self.url = url
        self.rss_file = rss_file
        self.rss_item_desc = self.make_rss_item_desc()
        self.svnlook = os.path.join(self.svn_path, "svnlook")
        (file, ext) = os.path.splitext(self.rss_file)
        self.pickle_file = file + ".pickle"
        self.rss_item = self.make_rss_item()
        self.rss = self.make_rss()
        
    def make_rss_item_desc(self):
        cmd = "svnlook info -r " + self.revision + " " + self.repos_path
        out, x, y = popen2.popen3(cmd)
        cmd_out = out.readlines()
        Author = "\nAuthor: " + cmd_out[0]
        Date = "Date: " + cmd_out[1]
        New_Revision = "Revision: " + self.revision
        Log = "Log: " + cmd_out[3]
        out.close()
        x.close()
        y.close()
        
        cmd = "svnlook changed -r " + self.revision + " " + self.repos_path
        out, x, y = popen2.popen3(cmd)
        cmd_out = out.readlines()
        changed_files = "Modified: \n"
        for item in cmd_out:
            changed_files = changed_files + item
        item_desc = Author + Date + New_Revision + "\n" + \
                    Log + changed_files
        out.close()
        x.close()
        y.close()
        
        return item_desc
        
    def pickle(self):
        s = StringIO()    
        pickle.dump(self.rss, s)
        f = open(self.pickle_file,"w")
        f.write (s.getvalue())
        f.close()

    def make_rss_item(self):
        """ Generate PyRSS2Gen Item from the commit info """
        item_title = "Revision " + self.revision
        item_link = url + "?rev=" + self.revision
        rss_item = PyRSS2Gen.RSSItem(title = item_title,
                                     link = item_link,
                                     description = self.make_rss_item_desc(),
                                     guid = PyRSS2Gen.Guid(item_link),
                                     pubDate = datetime.datetime.now())
        return rss_item

    def make_rss(self):
        """ Generate a PyRSS2Gen RSS2 object """
        if os.path.exists(self.pickle_file):
            f = open(self.pickle_file, "r")
            rss = pickle.load(f)
            f.close()
            if len(rss.items) == self.max_items :
                rss.items.pop()
            rss.items.insert(0, self.rss_item)
        else:
            rss_item = self.rss_item
            rss = PyRSS2Gen.RSS2(
                              title = "Foo's SVN Commits Feed",
                              link = "http://www.foo.com/project",
                              description = "The latest SVN commits",
                              lastBuildDate = datetime.datetime.now(),
                              items = [rss_item])

        return rss

svn2rss = SVN2RSS(svn_path, commit_rev, repos_path, url, rss_file)
rss = svn2rss.rss
svn2rss.pickle()
rss.write_xml(open(rss_file, "w"))



# -*- coding: utf-8 -*-

from pylons import config

class DefaultConfig:
    """ Default config values 
    Warning: do not change configuration here, change localconfig.py instead.
    Save localconfig.py.ini to the deploy directory as localconfig.py.
    """

    # authn_file: a .htpasswd style password file, used for pysvnmanager authentication.
    # You can change authn_file in <deploy>.ini file.

    # Unittest based on WebTest not initialized pylons.config
    if config.get('__file__') is None:
        import pylons.test
        wsgiapp = pylons.test.pylonsapp
        config = wsgiapp.config
    authn_file = config.get('authn_file', "") % {'here': config.get('here')}

    # authz_file: svn authz config file with administrative extension. (ossxp.com)
    # You can change authz_file in <deploy>.ini file.
    authz_file = config.get('authz_file', "") % {'here': config.get('here')}
    
    # Numbers of logs in each page.
    log_per_page = 10
    
    # svn repository root
    repos_root = config.get('repos_root', "") % {'here': config.get('here')}
    
    # pysvnmanager authentication method.
    # You can use htpasswd_login, or ldap_login, or both.
    # You can also implement your own auth module.
    #from pysvnmanager.model.auth.http import htpasswd_login
    #auth = [htpasswd_login, ] # single auth method

    # Htpasswd login parameters
    # Note: custom `authn_file' in your <deploy>.ini file

    #import ldap
    #from pysvnmanager.model.auth.ldap_login import ldap_login
    #auth = [ldap_login, htpasswd_login] # both
    #auth = [ldap_login] # ldap auth only
    
    # LDAP parameters
    ldap_uri = 'ldap://localhost'
    ldap_binddn = ''
    ldap_bindpw = ''
    ldap_base = 'dc=foo,dc=bar'
    ldap_scope = 2 # ldap.SCOPE_SUBTREE = 2
    ldap_filter = '(&(uid=%(username)s)(authorizedService=svn)(ossxpConfirmed=TRUE))'
    ldap_timeout = 10 # how long we wait for the ldap server [s]
    ldap_coding = 'utf-8' # coding used for ldap queries and result values
    ldap_start_tls = False
    



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys, locale
from svn import repos, fs, core
locale.setlocale(locale.LC_ALL, 'zh_CN.UTF8')

def test_authz(path):
  try:
    repos.svn_repos_authz_read(path, 1)
  except core.SubversionException, ( strerror, errorno ):
    if errorno == 0:
      errorno = 1
    return (errorno, strerror)
  except:
    return (1, sys.exc_info()[1])

  return (0, 0)

filename = sys.argv[1]
(errorno, strerror) = test_authz(filename)
if errorno:
  sys.stderr.write('Parse authz config file "%s" failed\n' % (filename))
  sys.stderr.write('Possible errors:\n')
  sys.stderr.write('  %s\n' % strerror)
  sys.exit(1)
else:
  sys.exit(0)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import logging

from pysvnmanager.lib.base import *
from pysvnmanager.lib.text import to_unicode
from pysvnmanager.model.svnauthz import *
from pysvnmanager.model import repos as _repos
from pysvnmanager.model import hooks as _hooks
from pylons.i18n import _, ungettext, N_

log = logging.getLogger(__name__)

class ReposController(BaseController):
    requires_auth = True
    
    def __init__(self):
        c.menu_active = "repos"
        try:
            self.authz = SvnAuthz(cfg.authz_file)
            self.login_as = session.get('user')
            # Used as checked in user to rcs file.
            self.authz.login_as = self.login_as

            self.own_reposlist = set(self.authz.get_manageable_repos_list(self.login_as))

            # self.all_reposlist is all available repositories in the ReposRoot directory.
            self.repos = _repos.Repos(cfg.repos_root)
            self.all_reposlist = set(self.repos.repos_list)

            self.reposlist_set = self.own_reposlist & self.all_reposlist
            self.reposlist_unexist = self.own_reposlist - self.all_reposlist

            self.is_super_user = self.authz.is_super_user(self.login_as)
            if self.is_super_user:
                self.reposlist_unset = self.all_reposlist - self.own_reposlist
            else:
                self.reposlist_unset = set()
        except Exception, e:
            import traceback
            g.catch_e = [unicode(e), traceback.format_exc(5) ]
            return

    def __before__(self, action=None):
        super(ReposController, self).__before__(action)
        if not self.own_reposlist and not self.is_super_user:
            return redirect(url(controller='security', action='failed'))
        diff = self.authz.differ()
        if diff:
            c.global_message = _('Some one maybe you, has modified the svn authz file by hands. Please save once to fix possible config error.') + "<blockquote>" + "<br>".join(diff.splitlines()) + "</blockquote>"
        
    def index(self):
        c.is_super_user = self.is_super_user
        return render('/repos/hooks.mako')

    def validate_repos(self, reposname):
        assert reposname in self.own_reposlist or self.is_super_user

    def init_repos_list(self):
        filter = request.params.get('filter')
        total = 0;
        msg = ''
 
        # get javascript code for top_form's role_selector
        msg += 'id[0]="%s";' % '...'
        msg += 'name[0]="%s";\n' % _("Please choose...")
        total += 1;
        for reposname in sorted(self.reposlist_set):
            if filter=='blank' and not self.repos.is_blank_svn_repos(reposname):
                continue
            msg += 'id[%d]="%s";' % (total, reposname)
            msg += 'name[%d]="%s";\n' % (total, reposname)
            total += 1;
        for reposname in sorted(self.reposlist_unset):
            if filter=='blank' and not self.repos.is_blank_svn_repos(reposname):
                continue
            msg += 'id[%d]="%s";' % (total, reposname)
            msg += 'name[%d]="%s";\n' % (total, reposname+' (!)')
            total += 1;
        msg += 'total=%d;\n' % total
        return msg

    def get_plugin_list(self):
        reposname = request.params.get('select')
        self.validate_repos(reposname)
        h = _hooks.Hooks(cfg.repos_root + '/' + reposname)
        total = 0;
        msg = ''
 
        # get javascript code for top_form's role_selector
        msg += 'id[0]="%s";' % '...'
        msg += 'name[0]="%s";\n' % _("Please choose...")
        total += 1;
        for name in h.unapplied_plugins:
            msg += 'id[%d]="%s";' % (total, name)
            msg += 'name[%d]="%s";\n' % (total, name + ': ' + h.plugins[name].name)
            total += 1;
        msg += 'total=%d;\n' % total
        
        return msg
    
    def get_installed_hook_form(self):
        reposname = request.params.get('select')
        self.validate_repos(reposname)
        h = _hooks.Hooks(cfg.repos_root + '/' + reposname)
        msg = ''
        if len(h.applied_plugins) > 0:
            msg += _("Installed hooks:")
            msg += "<br>\n"
            num = 0
            
            msg += "<table class='hidden'>\n"
            msg += "<tr><th align='left'></th>" + \
                    "<th align='left'>" + _("Id") + "</th>" + \
                    "<th align='left'>" + _("Plugin name") + "</th>" + \
                    "<th align='left'>" + _("Type") + "</th>" + \
                    "</tr>\n"
            for name in h.applied_plugins:
                msg += "<tr><td width='1' rolspan='2'>"
                msg += '<input type="checkbox" name="pluginid_%(num)d" value="%(plugin)s">' % {
                    'num': num, 'plugin': name, }
                msg += "</td>\n"
                msg += "<td><a href='#' onclick=\"show_hook_config_form('%s'); return false;\">" % name + name + "</a></td>\n"
                msg += "<td>" + h.plugins[name].name + "</td>\n"
                msg += "<td>" + h.plugins[name].get_type() + "</td>\n"
                msg += "</tr>\n"
                msg += "<tr><td></td><td colspan='3'>" + h.plugins[name].show_install_info() + "</td></tr>\n"
                num += 1
            msg += "</table>\n"
            msg += '<input type="submit" name="uninstall_hook" value="%s">\n' % _("Remove selected hooks")

        return msg
    
    def get_hook_setting_form(self):
        reposname  = request.params.get('repos')
        self.validate_repos(reposname)
        pluginname = request.params.get('plugin')
        h = _hooks.Hooks(cfg.repos_root + '/' + reposname)
        result  = "<input type='hidden' name='_repos' value='%s'>" % reposname
        result += "<input type='hidden' name='_plugin' value='%s'>" % pluginname
        result +=  h.plugins[pluginname].show_install_config_form()

        return result
    
    def setup_hook(self):
        try:
            d = request.params
            reposname = d.get("_repos")
            self.validate_repos(reposname)
            pluginname = d.get("_plugin")
            h = _hooks.Hooks(cfg.repos_root + '/' + reposname)
            plugin = h.plugins[pluginname]
            plugin.install(d)
        except Exception, e:
            result = "<div class='error'>" + _("Apply plugin '%(plugin)s' on '%(repos)s' Failed. Error message:<br>\n%(msg)s") % {
                        "plugin": pluginname, "repos":reposname, "msg": to_unicode(e) } + "</div>"
        else:
            result = "<div class='info'>" + _("Apply plugin '%(plugin)s' on '%(repos)s' success.") % {
                        "plugin": pluginname, "repos":reposname} + "</div>"
        return result
    
    def uninstall_hook(self):
        plugin_list=[]
        d = request.params
        reposname = d.get("_repos")
        self.validate_repos(reposname)
        for i in d.keys():
            if "pluginid_" in i:
                plugin_list.append(d[i])

        if plugin_list:
            log.debug("plugin_list:" + ','.join(plugin_list))
            try:
                hookobj = _hooks.Hooks(cfg.repos_root + '/' + reposname)
                for pluginname in plugin_list:
                    hookobj.plugins[pluginname].reload()
                    hookobj.plugins[pluginname].uninstall()
                    log.info("my delete plugin %s, %s" % (pluginname, hookobj.plugins[pluginname].name))
            except Exception, e:
                result = "<div class='error'>" + _("Delete plugin '%(plugin)s' on '%(repos)s' Failed. Error message:<br>\n%(msg)s") % {
                        "plugin": ", ".join(plugin_list), "repos":reposname, "msg": to_unicode(e) } + "</div>"
            else:
                result = "<div class='info'>" + _("Delete plugin '%(plugin)s' on '%(repos)s' success.") % {
                        "plugin": ", ".join(plugin_list), "repos":reposname} + "</div>"
        else:
            result = "<div class='error'>" + _("No plugin has been deleted for '%(repos)s'.") % {"repos":reposname} + "</div>"
        return result

    def create_submit(self):
        try:
            d = request.params
            reposname = d.get("reposname")
            self.validate_repos(reposname)
            self.repos.create(reposname)
        except Exception, e:
            result = "<div class='error'>" + _("Create repository '%(repos)s' Failed. Error message:<br>\n%(msg)s") % {
                        "repos":reposname, "msg": to_unicode(e) } + "</div>"
        else:
            result = "<div class='info'>" + _("Create repository '%(repos)s' success.") % {"repos":reposname} + "</div>"
        return result
        
    def create(self):
        if not self.is_super_user:
            return redirect(url(controller='security', action='failed'))
        return render('/repos/create.mako')


    def remove_submit(self):
        try:
            d = request.params
            reposname = d.get("repos_list")
            self.validate_repos(reposname)
            self.repos.delete(reposname)
        except Exception, e:
            result = "<div class='error'>" + _("Delete repository '%(repos)s' Failed. Error message:<br>\n%(msg)s") % {
                        "repos":reposname, "msg": to_unicode(e) } + "</div>"
        else:
            result = "<div class='info'>" + _("Delete blank repository '%(repos)s' success.") % {"repos":reposname} + "</div>"
        return result
    
    def remove(self):
        if not self.is_super_user:
            return redirect(url(controller='security', action='failed'))
        return render('/repos/remove.mako')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008,2009 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from pysvnmanager.hooks.plugins import *
from pysvnmanager.hooks.plugins import _
import webhelpers

class TracPostCommit(PluginBase):

    # Brief name for this plugin.
    name = _("Trac integration with subversion's post commit hook.")
    
    # Both description and detail are reStructuredText format. 
    # Reference about reStructuredText: http://docutils.sourceforge.net/docs/user/rst/quickref.html

    # Short description for this plugin.
    description = _("Integrate subversion with trac: Commit log of subversion appends to trac tickets if subversion commit log contains ticket id.")

    # Long description for this plugin.
    detail = ""
    
    # Hooks-plugin type: T_START_COMMIT, ..., T_POST_UNLOCK
    type = T_POST_COMMIT
    
    # Plugin config option/value in config ini file.
    key_switch     = "trac_post_commit_enabled"
    key_trac_env   = "trac_env"
    key_trac_repos_name = "trac_repos_name"
    key_trac_fixed_status = "trac_fixed_status"
    
    section = 'trac'
    
    def enabled(self):
        """
        Return True, if this plugin has been installed.
        Simply call 'has_config()'.
        """
        return self.has_config(self.key_switch)
    
    def install_info(self):
        """
        Show configurations if plugin is already installed.
        
        return reStructuredText.
        reST reference: http://docutils.sourceforge.net/docs/user/rst/quickref.html
        """
        result = self.description
        if self.enabled():
            result += "\n\n"
            result += "**" + _("Current configuration") + "**\n\n"
            if self.get_config(self.key_switch) == "yes":
                result += "- " + _("Trac post commit hook is enabled.")
            else:
                result += "- " + _("Trac post commit hook is disabled.")
            result += "\n"
            result += "- " + _("Trac environment location: ") + self.get_config(self.key_trac_env)
            result += "\n"
            result += "- " + _("Repository name in trac: ") + self.get_config(self.key_trac_repos_name) or '*default*'
            result += "\n"
            result += "- " + _("Fixed ticket's status: ") + self.get_config(self.key_trac_fixed_status) or '*default*'
            
        return result
    
    def install_config_form(self):
        """
        This method will be called to build setup configuration form.
        If this plugin needs parameters, provides form fields here.
        Any html and javascript are welcome.
        """
        if self.get_config(self.key_switch)=="no":
            enable_checked  = ""
            disable_checked = "checked"
        else:
            enable_checked  = "checked"
            disable_checked = ""

        result = ""
        result += "<p><strong>%s</strong></p>" % _("Fill this form")
        result += "<blockquote>"
        result += "\n<dl>"
        result += "\n<dt>"
        result += _("Enable trac post commit hook: ")
        result += "\n<dd>"
        result += "<input type='radio' name='switch' value='yes' " + \
                enable_checked  + ">" + _("Enable") + "&nbsp;"
        result += "<input type='radio' name='switch' value='no' " + \
                disable_checked + ">" + _("Disable")
        result += "\n<dt>"
        result += _("Trac environment location: ")
        result += "\n<dd>"
        result += "<input type='text' name='trac_env' size='50' value=\"%s\">" % \
                webhelpers.util.html_escape(self.get_config(self.key_trac_env))
        result += "\n<dt>"
        result += _("Repository name in trac (default is blank): ")
        result += "\n<dd>"
        result += "<input type='text' name='trac_repos_name' size='20' value=\"%s\">" % \
                webhelpers.util.html_escape(self.get_config(self.key_trac_repos_name))
        result += "\n<dt>"
        result += _("Fixed ticket status (default is closed): ")
        result += "\n<dd>"
        result += "<input type='text' name='trac_fixed_status' size='10' value=\"%s\">" % \
                webhelpers.util.html_escape(self.get_config(self.key_trac_fixed_status))
        result += "\n</dl>"
        result += "</blockquote>"
        return result
        
    def uninstall(self):
        """
        Uninstall hooks-plugin from repository.
        Simply call 'unset_config()' and 'save()'.
        """
        self.unset_config(self.key_switch)
        self.unset_config(self.key_trac_env)
        self.unset_config(self.key_trac_repos_name)
        self.unset_config(self.key_trac_fixed_status)
        self.save()
    
    def install(self, params=None):
        """
        Install hooks-plugin from repository.
        Simply call 'set_config()' and 'save()'.
        
        Form fields in setup_config() will pass as params.
        """
        switch = params.get('switch', 'yes')
        if switch != 'yes':
            switch = 'no'
        trac_env = params.get('trac_env')
        trac_repos_name = params.get('trac_repos_name')
        trac_fixed_status = params.get('trac_fixed_status')
        log.debug("trac post commit setting: %s, %s" % (trac_env, trac_repos_name))
        if not trac_env:
            raise Exception('Trac environment not defined!')
        elif not trac_env.startswith('/'):
            raise Exception('Trac environment isn\'t a absolute path! It must begin with /.')
        self.set_config(self.key_switch, switch)
        self.set_config(self.key_trac_env, trac_env)
        self.set_config(self.key_trac_repos_name, trac_repos_name)
        self.set_config(self.key_trac_fixed_status, trac_fixed_status)
        self.save()
        
def execute(repospath=""):
    """
    Generate and return a hooks plugin object

    @param request: repos full path
    @rtype: Plugin
    @return: Plugin object
    """
    return TracPostCommit(repospath)



#!/usr/bin/python
# -*- coding: utf-8 -*-

import sys
import os
import os.path
from svn import repos, fs, delta, core


### DEAR USER:  Please populate the test_props() and test_path_change()
### to do your bidding.


def test_props(props):
  """Validate the PROPS (a dictionary mapping property names to
  values) set on the transaction.  Return 0 if all is well, non-zero
  otherwise."""

  ### Test the transaction (revision-to-be) properties.  If there is
  ### bogosity, write to sys.stderr and return non-zero.

  return 0


def test_path_change(path, change):
  """Validate the CHANGE made to PATH in the transaction.  Return 0
  if all is well, non-zero otherwise."""
  
  # The svn_node_kind of the path.
  item_kind = change.item_kind

  # Non-zero iff properties of this path were changed.
  prop_changes = change.prop_changes

  # Non-zero iff path is a file, and its text was changed.
  text_changed = change.text_changed

  # The location of the previous revision of this resource, if any.
  base_path = change.base_path
  base_rev = change.base_rev

  # Non-zero iff this change represents an addition (see
  # base_path/base_rev for whether it was an addition with history).
  added = change.added

  ### Test the path change as you see fit.  If there is bogosity,
  ### write to sys.stderr and return non-zero.

  return 1


def main(pool, repos_dir, txn):
  # Construct a ChangeCollector to fetch our changes.
  fs_ptr = repos.svn_repos_fs(repos.svn_repos_open(repos_dir, pool))
  root = fs.txn_root(fs.open_txn(fs_ptr, txn, pool), pool)
  cc = repos.ChangeCollector(fs_ptr, root, pool)

  # Call the transaction property validator.  Might as well get the
  # cheap checks outta the way first.
  retval = test_props(cc.get_root_props())
  if retval:
    return retval

  # Generate the path-based changes list.
  e_ptr, e_baton = delta.make_editor(cc, pool)
  repos.svn_repos_replay(root, e_ptr, e_baton, pool)

  # Call the path change validator.
  changes = cc.get_changes()
  paths = changes.keys()
  paths.sort(lambda a, b: core.svn_path_compare_paths(a, b))
  for path in paths:
    change = changes[path]
    retval = test_path_change(path, change)
    if retval:
      return retval
    
  return 0


def _usage_and_exit():
  sys.stderr.write("USAGE: %s REPOS-DIR TXN-NAME\n" % (sys.argv[0]))
  sys.exit(1)


if __name__ == '__main__':
  if len(sys.argv) < 3:
    _usage_and_exit()
  sys.exit(core.run_app(main, sys.argv[1], sys.argv[2]))
  



#!/usr/bin/python
# -*- coding: utf-8 -*-

"""
check-mime-type.py: check that every added file has the
svn:mime-type property set and every added file with a mime-type
matching text/* also has svn:eol-style set. If any file fails this
test the user is sent a verbose error message suggesting solutions and
the commit is aborted.

Usage: commit-mime-type-check.pl REPOS TXN-NAME

Rewrite from check-mime-type.pl, by Jiang Xin<WorldHello.net.AT.gmail.com>
"""

__revision__ = '$Id: commit_log_check.py 513 2006-05-06 17:12:03Z jiangxin $'

import sys, os, re, string, locale

if os.name == 'nt':
    SVNLOOK = 'C:/Apps/Subversion/bin/svnlook.exe'
else:
    SVNLOOK = '/opt/svn/bin/svnlook'

os.environ['LANG'] = os.environ['LC_ALL'] = 'zh_CN.UTF8'

MIN_LENGTH = 5

def main(repos, txn, force=""):
    """main entry point"""
    
    if force == "no":
        force = False 
    else:
        force = True

    files_added = []
    cmd = '%s changed -t "%s" "%s"' % (SVNLOOK, txn, repos)
    padd = re.compile(r'^A.  (.*[^/])$')

    for line in os.popen(cmd, 'r').readlines():
        match = padd.match( line.rstrip("\n") );
        if match:
            groups = match.groups()
            if len(groups) == 1:
                files_added.append( groups[0] );

    pmime = re.compile(r'\s*svn:mime-type : (\S+)')
    peol  = re.compile(r'\s*svn:eol-style : (\S+)')
    ptext = re.compile(r'^text/')
    pspecial = re.compile(r'\s*svn:special : (\S+)')
    
    errmsg = []
    for path in files_added:
        cmd = '%s proplist -t "%s" "%s" --verbose "%s"' % (SVNLOOK, txn, repos, path)
        mime_type = ''
        eol_style = ''
        check_mime = True
        
        for line in os.popen(cmd, 'r').readlines():
            if pmime.match(line):
                mime_type = pmime.match(line).group(1)
            elif pspecial.match(line):
                check_mime = False
            if peol.match(line):
                eol_style = peol.match(line).group(1)
        
        if check_mime:
            if mime_type == "" or ptext.match(mime_type):
                if eol_style == '':
                    ## check if crlf in file contents
                    if not force:
                        if crlf_in_file(txn, repos, path):
                            errmsg.append( "CRLF (DOS style EOL) in file: %s" % path.decode('utf-8','replace').encode('utf-8','replace') )
                        else:
                            continue
                    if mime_type == "":
                        errmsg.append( "%s : 属性 svn:mime-type 或者 svn:eol-style 没有设置" % path.decode('utf-8','replace').encode('utf-8','replace') )
                    else:
                        errmsg.append( "%s : svn:mime-type=%s 但是 svn:eol-style 没有设置" % (path.decode('utf-8','replace').encode('utf-8','replace'), mime_type) )

    if len( errmsg ) > 0:
        die( errmsg )

def crlf_in_file(txn, repos, path):
    cmd = '%s cat -t "%s" "%s" "%s"' % (SVNLOOK, txn, repos, path)
    buff = os.popen(cmd, 'r').read(1024)
    if '\r' in buff:
        return True
    else:
        return False
 
def die(msg):
    """
    Write verbose mesage, and exit
    """

    sys.stderr.write( "\n%s\n" % ("="*20) )
    sys.stderr.write( string.join(msg, '\n') )
    sys.stderr.write( "\n%s\n" % ("="*20) )
    
    sys.stderr.write("""\n
管理员已经启用换行符属性检查。每一个新添加的文件必须
指定换行符。如果 svn:mime-type 属性为文本文件，则
必须设置 svn:eol-style 属性。

对于二进制文件，执行如下命令：
svn propset svn:mime-type application/octet-stream path/of/file

对于文本文件，可以执行如下命令：
svn propset svn:mime-type text/plain path/of/file
svn propset svn:eol-style native path/of/file

为了避免每次添加文件手动设置，可以启用自动属性设置。
需要修改文件 ~/.subversion/config (Unix平台)。
打开 auto-props 设置，并设置扩展名和属性的对应关系。
详细设置，参见 Subversion 参考:
(http://svnbook.red-bean.com/), Chapter 7, Properties section,
Automatic Property Setting subsection.""")
    
    sys.exit(1)


if __name__ == '__main__':
    if len(sys.argv) < 3:
        sys.stderr.write("Usage: %s REPOS TXN\n" % (sys.argv[0]))
    else:
        main(sys.argv[1], sys.argv[2], sys.argv[3:] and sys.argv[3] or '')



# -*- coding: utf-8 -*-
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from pysvnmanager.tests import *

class TestLogsController(TestController):

    def test_index(self):
        response = self.app.get(url(controller='logs', action='index'))
        # Test response...



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

"""Subversion repos hooks plugin.

Basic classes used for Subversion hooks management.
"""

import re
import sys
import os
import StringIO
import logging
log = logging.getLogger(__name__)

from pylons import config
config_path = config["here"] + '/config'
if config_path not in sys.path:
    sys.path.insert(0, config_path)
from localconfig import LocalConfig as cfg

sys.path.insert(0,os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
from pysvnmanager.hooks import plugins
from pysvnmanager.model import repos

# i18n works only as pysvnmanager (a pylons app) model.
if not config.has_key('unittest'):
    from pylons.i18n import _
else:
    def _(message): return message

#reload(sys) # in Python2.5, method sys.setdefaultencoding 
            #will be delete after initialize. we need reload it.
#sys.setdefaultencoding('utf-8')

class Hooks:
    
    def __init__(self, repos_path):
        self.__repos_path = os.path.abspath(repos_path)
        self.__repos_root = os.path.dirname(self.__repos_path)
        self.__repos_name = os.path.basename(self.__repos_path)
        self.repos = repos.Repos(self.__repos_root)
        assert self.repos.is_svn_repos(self.__repos_name)
        
        self.plugins = {}
        for m in plugins.modules:
            self.plugins[m] = plugins.getHandler(m)(self.__repos_path)
            self.plugins[m].id = m
        self.pluginnames = [ m.id for m in sorted(self.plugins.values()) ]
    
    def __get_applied_plugins(self):
        return [ m for m in self.pluginnames if self.plugins[m].enabled()]
    
    applied_plugins = property(__get_applied_plugins)

    def __get_unapplied_plugins(self):
        return [ m for m in self.pluginnames if not self.plugins[m].enabled()]
    
    unapplied_plugins = property(__get_unapplied_plugins)
    
    def __get_repos_root(self):
        return self.__repos_root
    
    repos_root = property(__get_repos_root)
    
    def __get_repos_name(self):
        return self.__repos_name
    
    repos_name = property(__get_repos_name)
        
    def __get_repos_path(self):
        return self.__repos_path
    
    repos_path = property(__get_repos_path)
        
 

if __name__ == '__main__':
    import doctest
    doctest.testmod()



# -*- coding: utf-8 -*-
import logging
import sys
import ldap
log = logging.getLogger(__name__)

def ldap_login(username, password, config):
    """ get authentication data from form, authenticate against LDAP (or Active
        Directory), fetch some user infos from LDAP and create a user object
        for that user. The session is kept by the moin_session auth plugin.
    """
    verbose = getattr(config, 'ldap_verbose', False)

    # we require non-empty password as ldap bind does a anon (not password
    # protected) bind if the password is empty and SUCCEEDS!
    if not password or not username or not config:
        return False

    try:
        try:
            u = None
            dn = None
            coding = getattr(config, 'ldap_coding', 'utf-8')
            log.debug("LDAP: Setting misc. options...")
            ldap.set_option(ldap.OPT_PROTOCOL_VERSION, ldap.VERSION3) # ldap v2 is outdated
            ldap.set_option(ldap.OPT_REFERRALS, getattr(config, 'ldap_referrals', 0))
            ldap.set_option(ldap.OPT_NETWORK_TIMEOUT, getattr(config, 'ldap_timeout',10))

            starttls = getattr(config, 'ldap_start_tls', False)
            if hasattr(ldap, 'TLS_AVAIL') and ldap.TLS_AVAIL:
                for option, value in (
                    (ldap.OPT_X_TLS_CACERTDIR, getattr(config, 'ldap_tls_cacertdir', '')),
                    (ldap.OPT_X_TLS_CACERTFILE, getattr(config, 'ldap_tls_cacertfile', '')),
                    (ldap.OPT_X_TLS_CERTFILE, getattr(config, 'ldap_tls_certfile', '')),
                    (ldap.OPT_X_TLS_KEYFILE, getattr(config, 'ldap_tls_keyfile', '')),
                    (ldap.OPT_X_TLS_REQUIRE_CERT, getattr(config, 'ldap_tls_require_cert', ldap.OPT_X_TLS_NEVER)),
                    (ldap.OPT_X_TLS, starttls),
                    #(ldap.OPT_X_TLS_ALLOW, 1),
                ):
                    if value:
                        ldap.set_option(option, value)

            server = getattr(config, 'ldap_uri', 'ldap://localhost')
            if verbose: log.debug("LDAP: Trying to initialize %r." % server)
            l = ldap.initialize(server)
            if verbose: log.debug("LDAP: Connected to LDAP server %r." % server)

            if starttls and server.startswith('ldap:'):
                if verbose: log.debug("LDAP: Trying to start TLS to %r." % server)
                try:
                    l.start_tls_s()
                    if verbose: log.debug("LDAP: Using TLS to %r." % server)
                except (ldap.SERVER_DOWN, ldap.CONNECT_ERROR), err:
                    if verbose: log.debug("LDAP: Couldn't establish TLS to %r (err: %s)." % (server, str(err)))
                    raise

            # you can use %(username)s and %(password)s here to get the stuff entered in the form:
            ldap_binddn = getattr(config, 'ldap_binddn', '') % locals()
            ldap_bindpw = getattr(config, 'ldap_bindpw', '') % locals()
            l.simple_bind_s(ldap_binddn.encode(coding), ldap_bindpw.encode(coding))
            if verbose: log.debug("LDAP: Bound with binddn %r" % ldap_binddn)

            # you can use %(username)s here to get the stuff entered in the form:
            filterstr = getattr(config, 'ldap_filter', '(&(uid=%(username)s)(authorizedService=svn)(ossxpConfirmed=TRUE))') % locals()

            #attrs = [getattr(config, attr) for attr in [
            #                         'ldap_email_attribute',
            #                         'ldap_aliasname_attribute',
            #                         'ldap_surname_attribute',
            #                         'ldap_givenname_attribute',
            #                         ] if getattr(config, attr) is not None]
            attrs = ['cn', 'sn']

            if verbose: log.debug("LDAP: Searching %r" % filterstr)
            lusers = l.search_st(config.ldap_base, getattr(config, 'ldap_scope', ldap.SCOPE_SUBTREE), filterstr.encode(coding),
                                 attrlist=attrs, timeout=getattr(config, 'ldap_timeout', 10))
            # we remove entries with dn == None to get the real result list:
            lusers = [(dn, ldap_dict) for dn, ldap_dict in lusers if dn is not None]
            if verbose:
                for dn, ldap_dict in lusers:
                    log.debug("LDAP: dn:%r" % dn)
                    for key, val in ldap_dict.items():
                        log.debug("    %r: %r" % (key, val))

            result_length = len(lusers)
            if result_length != 1:
                if result_length > 1:
                    log.debug("LDAP: Search found more than one (%d) matches for %r." % (result_length, filterstr))
                if result_length == 0:
                    if verbose: log.debug("LDAP: Search found no matches for %r." % (filterstr, ))
                return False # if ldap returns unusable results, we veto the user and don't let him in

            dn, ldap_dict = lusers[0]
            if verbose: log.debug("LDAP: DN found is %r, trying to bind with pw" % dn)
            l.simple_bind_s(dn, password.encode(coding))
            if verbose: log.debug("LDAP: Bound with dn %r (username: %r)" % (dn, username))


        except ldap.INVALID_CREDENTIALS, err:
            log.debug("LDAP: invalid credentials (wrong password?) for dn %r (username: %r)" % (dn, username))
            return False # if ldap says no, we veto the user and don't let him in

    except:
        import traceback
        info = sys.exc_info()
        log.debug("LDAP: caught an exception, traceback follows...")
        log.debug(''.join(traceback.format_exception(*info)))
        return False # something went completely wrong, in doubt we veto the login

    return True




# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import logging

from pysvnmanager.lib.base import *
from pysvnmanager.model.svnauthz import *
from pysvnmanager.model import repos as _repos
from pylons.i18n import _, ungettext, N_

log = logging.getLogger(__name__)

class CheckController(BaseController):
    requires_auth = True

    def __init__(self):
        c.menu_active = "check"
        try:
            self.authz = SvnAuthz(cfg.authz_file)
            self.login_as = session.get('user')
            # Used as checked in user to rcs file.
            self.authz.login_as = self.login_as
            self.reposlist = self.authz.get_manageable_repos_list(self.login_as)
            if self.authz.is_super_user(self.login_as):
                for i in _repos.Repos(cfg.repos_root).repos_list:
                    if i not in self.reposlist:
                        self.reposlist.append(i)
                self.reposlist = sorted(self.reposlist)
        except Exception, e:
            import traceback
            g.catch_e = [unicode(e), traceback.format_exc(5) ]
            return

    def __before__(self, action=None):
        super(CheckController, self).__before__(action)
        if not self.reposlist:
            return redirect(url(controller='security', action='failed'))
        diff = self.authz.differ()
        if diff:
            c.global_message = _('Some one maybe you, has modified the svn authz file by hands. Please save once to fix possible config error.') + "<blockquote>" + "<br>".join(diff.splitlines()) + "</blockquote>"
        
    def index(self):
        c.reposlist = self.reposlist
        c.userlist = map(lambda x:x.uname, self.authz.grouplist)
        c.userlist.extend(map(lambda x:x.uname, self.authz.aliaslist))
        c.userlist.extend(map(lambda x:x.uname, self.authz.userlist))
        c.pathlist = []
        return render('/check/index.mako')
    
    def access_map(self):
        msg = ""
        d = request.params
        
        if d.get('userinput') == 'manual':
            username = d.get('username')
        else:
            username = d.get('userselector')

        if d.get('reposinput') == 'manual':
            repos = d.get('reposname')
        else:
            repos = d.get('reposselector')

        if d.get('pathinput') == 'manual':
            path = d.get('pathname')
        else:
            path = d.get('pathselector')

        abbr = d.get('abbr', 'False')
        if abbr.lower() == 'true' or abbr == '1':
            abbr = True
        else:
            abbr = False
        
        if username != '...' and repos != '...':
            if repos == '*':
                repos = self.reposlist
            if not '/' in self.reposlist:
                if not repos or \
                    ( isinstance(repos, basestring) and not repos in self.reposlist ):
                    return _("Permission denied.")
            if path and path != "*":
                msglist = self.authz.get_path_access_msgs(username, repos, path, abbr=abbr)
                msg += "<div id='acl_path_msg'>" + "<br>\n".join(msglist) + "</div>"

            msg += "<pre>"+ "\n".join(self.authz.get_access_map_msgs(username, repos, abbr=abbr))+"</pre>"

        return msg
        
    def get_auth_path(self, repos=None, type=None, path=None):
        total = 0;
        msg = ''
        d = request.params
        reposname = d.get('repos')
        repos = self.authz.get_repos(reposname)
        if not repos:
            return msg;

        # get javascript code for top_form's role_selector
        msg += 'id[0]="%s";' % '...'
        msg += 'name[0]="%s";\n' % _("Please choose...")
        total += 1;
        for path in repos.path_list:
            msg += 'id[%d]="%s";' % (total, path)
            msg += 'name[%d]="%s";\n' % (total, path)
            total += 1;
        msg += 'total=%d;\n' % total
        
        return msg



# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import logging

from pysvnmanager.lib.base import *
from pysvnmanager.lib.text import to_unicode
from pysvnmanager.model import rcsbackup as _rcs
from pysvnmanager.model.svnauthz import *
from pylons.i18n import _, ungettext, N_

log = logging.getLogger(__name__)

class LogsController(BaseController):
    requires_auth = True
    
    def __init__(self):
        c.menu_active = "logs"
        try:
            self.authz = SvnAuthz(cfg.authz_file)
            self.login_as = session.get('user')
            self.rcslog = _rcs.RcsLog(cfg.authz_file)
            # Default logs per page is 10
            self.rcslog.log_per_page = cfg.log_per_page
            self.is_super_user = self.authz.is_super_user(self.login_as)
            self.own_reposlist = set(self.authz.get_manageable_repos_list(self.login_as))
        except Exception, e:
            import traceback
            g.catch_e = [unicode(e), traceback.format_exc(5) ]
            return
    
    def __before__(self, action=None):
        super(LogsController, self).__before__(action)
        if not self.own_reposlist and not self.is_super_user:
            return redirect(url(controller='security', action='failed'))
        diff = self.authz.differ()
        if diff:
            c.global_message = _('Some one maybe you, has modified the svn authz file by hands. Please save once to fix possible config error.') + "<blockquote>" + "<br>".join(diff.splitlines()) + "</blockquote>"
    
    def index(self):
        c.display = self.__get_log_display(1)
        return render('/logs/index.mako')

    def paginate(self):
        d = request.params
        page = int(d.get('page', '1'))
        return self.__get_log_display(page)

        
    def __get_log_display(self, current=1):
        logs = self.rcslog.get_page_logs(current)
        if not logs:
            return ""
        paginate = self.__get_paginate(current)
        
        buff  = '<div>%s</div>' % paginate
        buff +='''
<div>
<table>
<tr>
    <th>%(rev)s</th>
    <th>%(who)s</th>
    <th>%(when)s</th>
    <th>%(why)s</th>
    <th>%(comp)s</th>
</tr>''' % {'rev': _("Rev"), 
            'who': _("Who"), 
            'when': _("When"), 
            'why': _("Why"),
            'comp': _("Compare"),
            }
        
        for i in range(len(logs)-1, -1, -1):
            buff += '''
<tr>
    <td>%(rev)s</t>
    <td>%(who)s</td>
    <td>%(when)s</td>
    <td>%(why)s</td>
    <td>
        <input type="radio" name="left" value="%(rev)s">
        <input type="radio" name="right" value="%(rev)s">
    </td>
</tr>''' % {'rev' : logs[i].get('revision',''), 
            'who' : logs[i].get('author',''), 
            'when': logs[i].get('date',''), 
            'why' : h.link_to(logs[i].get('log',''), \
                              url(controller='logs', action='view', id=logs[i].get('revision','')), \
                              onclick="window.open(this.href,'view_logs','location=0,toolbar=0,width=780,height=580');return false;"
                              ), 
            }
        
        buff += '''
</table></div>
<div>%s</div>''' % paginate

        return buff
    
    def __get_paginate(self, current=1):
        def link(i):
            return '<a href="#" onclick="paginate(%d)">%d</a>' % (i,i)
        
        total_page = self.rcslog.total_page
        if total_page < 2:
            return ""

        if current < 1:
            current = 1
        if current > total_page:
            current = total_page
        
        sep = " "
        buff = _("Page: ")
        
        i=1
        while True:
            if i > total_page:
                break
            if i == current:
                buff += '%d%s' % (i, sep)
                i+=1
            elif i == 1 or i == total_page or i == current-1 or i == current+1:
                buff += '%s%s' % (link(i), sep)
                i+=1
            elif i < current-1:
                buff += '...%s' % sep
                i = current-1
            elif i > current+1:
                buff += '...%s' % sep
                i = total_page
            else:
                i+=1

        return buff 
    
    def compare(self):
        d = request.params
        left  = d.get('left', '')
        right = d.get('right', '')
        if not left or not right:
            return ""
        if left == right:
            return ""
        
        buff = '''<h2>%(title)s
<input type="radio" name="left" value="%(left)s">%(left)s
<input type="radio" name="right" value="%(right)s">%(right)s
</h2>
''' % {'title': _("Compares between"),
       'left' : left,
       'right': right}

        buff += "<pre>%s</pre>" % self.rcslog.differ(left, right)
        return buff


    def view(self, id):
        assert id and isinstance(id, basestring)
        c.contents = self.rcslog.cat(id)
        c.log = self.rcslog.get_logs(id, id)[0]
        if self.rcslog.head != id and self.is_super_user:
            c.rollback_enabled = True
        else:
            c.rollback_enabled = False
        return render('/logs/view.mako')
    
    def rollback(self, id):
        assert self.is_super_user
        log_message = _("Rollback successfully to revision: %s") % id
        try:
            assert id and isinstance(id, basestring)
            self.rcslog.restore(id)
            self.rcslog.backup(comment=log_message, user=self.login_as)
        except Exception, e:
            msg = to_unicode(e)
            c.msg = _("Rollback failed: %s") % msg
        else:
            c.msg = log_message

        return render('/logs/rollback.mako')
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

import os
import sys
sys.path.insert(0,os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))

from pysvnmanager.lib.base import *
from pysvnmanager.tests import *
from pysvnmanager import model
from pysvnmanager.model import repos
from pysvnmanager.model import hooks
from pysvnmanager.hooks import plugins

import pylons.test

import StringIO
from pprint import pprint

class TestRepos(TestController):
    
    def __init__(self, *args):
        # self.repos_root = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/svnroot.test'
        wsgiapp = pylons.test.pylonsapp
        config = wsgiapp.config
        self.repos_root = config.get('repos_root', "") % {'here': config.get('here')}
        self.repos = repos.Repos(self.repos_root)
        super(TestRepos, self).__init__(*args)

    def setUp(self):
        #print '+'*40, 'setup'
        pass

    def tearDown(self):
        #print '+'*40, 'teardown'
        pass
    
    def testReposCreate(self):
        self.assertRaises(Exception, self.repos.create, 'repos3')
        try:
            self.repos.delete('repos3')
            self.repos.create('repos3')
            self.assert_(sorted(self.repos.repos_list) == [u'project1', u'project2', u'repos3'], self.repos.repos_list)
        except ImportError:
            pass

    def testReposDelete(self):
        try:
            self.repos.delete('project1')
        except Exception, e:
            self.assert_(str(e) == 'Repos project1 is not a blank repository.', str(e))
    
    def testReposRoot(self):
        repos.Repos('/tmp')
        self.assertRaises(Exception, repos.Repos, '/tmp/svnroot.noexists')

    def testReposlist(self):
        self.assert_(sorted(self.repos.repos_list) == ['project1', 'project2', 'repos3'], u','.join(self.repos.repos_list).encode('utf-8'))

    def testSvnVersion(self):
        svnversion = self.repos.svnversion()
        self.assert_(svnversion[0]!='', svnversion[0])
        self.assert_(svnversion[1]!='', svnversion[1])

class TestReposPlugin(TestController):
    
    def __init__(self, *args):
        # self.repos_root = os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))) + '/svnroot.test'
        wsgiapp = pylons.test.pylonsapp
        config = wsgiapp.config
        self.repos_root = config.get('repos_root', "") % {'here': config.get('here')}
        super(TestReposPlugin, self).__init__(*args)

    def setUp(self):
        #print '+'*40, 'setup'
        pass

    def tearDown(self):
        #print '+'*40, 'teardown'
        pass

    def testPluginList(self):
        self.assert_('CaseInsensitive' in plugins.modules, plugins.modules)
        self.assert_('EolStyleCheck' in plugins.modules, plugins.modules)
      
    def testPluginImport(self):
        self.assertRaises(Exception,  plugins.getHandler("CaseInsensitive"), "")
        module_ci = plugins.getHandler("CaseInsensitive")(self.repos_root + '/project1')
        self.assert_(module_ci.name=="Detect case-insensitive filename clashes", module_ci.name)
        self.assert_(module_ci.description!="", module_ci.description)
      
    def testPluginSetting(self):
        m = plugins.getHandler("CaseInsensitive")(self.repos_root + '/project1')
        self.assert_(m.enabled()==False)
        m.install()
        self.assert_(m.enabled()==True)
        m.uninstall()
        self.assert_(m.enabled()==False)
      
    def testHooks(self):
        # self.repos_root is not a repository.
        self.assertRaises(AssertionError, hooks.Hooks, self.repos_root)
        
        myhooks = hooks.Hooks(self.repos_root + '/project1')
        self.assert_('CaseInsensitive' in myhooks.pluginnames, myhooks.pluginnames)
        self.assert_('EolStyleCheck' in myhooks.pluginnames, myhooks.pluginnames)
        
        self.assert_('CaseInsensitive' in myhooks.unapplied_plugins, myhooks.unapplied_plugins)
        self.assert_('EolStyleCheck' in myhooks.unapplied_plugins, myhooks.unapplied_plugins)
        
        m = myhooks.plugins['CaseInsensitive']
        self.assert_(m.name=="Detect case-insensitive filename clashes", m.name)
        self.assert_(m.description!="", m.description)
    
    def testHooksSetting(self):
        myhooks = hooks.Hooks(self.repos_root + '/project1')

        m = myhooks.plugins['CaseInsensitive']
        self.assert_(m.enabled()==False)
        self.assert_('CaseInsensitive' not in myhooks.applied_plugins, myhooks.applied_plugins)
        self.assert_('EolStyleCheck' not in myhooks.applied_plugins, myhooks.applied_plugins)
        self.assert_('CaseInsensitive' in myhooks.unapplied_plugins, myhooks.unapplied_plugins)
        self.assert_('EolStyleCheck' in myhooks.unapplied_plugins, myhooks.unapplied_plugins)

        m.install()
        self.assert_(m.enabled()==True)
        self.assert_('CaseInsensitive' in myhooks.applied_plugins, myhooks.applied_plugins)
        self.assert_('EolStyleCheck' not in myhooks.applied_plugins, myhooks.applied_plugins)
        self.assert_('CaseInsensitive' not in myhooks.unapplied_plugins, myhooks.unapplied_plugins)
        self.assert_('EolStyleCheck' in myhooks.unapplied_plugins, myhooks.unapplied_plugins)

        m.uninstall()
        self.assert_(m.enabled()==False)
        self.assert_('CaseInsensitive' not in myhooks.applied_plugins, myhooks.applied_plugins)
        self.assert_('EolStyleCheck' not in myhooks.applied_plugins, myhooks.applied_plugins)
        self.assert_('CaseInsensitive' in myhooks.unapplied_plugins, myhooks.unapplied_plugins)
        self.assert_('EolStyleCheck' in myhooks.unapplied_plugins, myhooks.unapplied_plugins)


if __name__ == '__main__': 
    import unittest
    unittest.main()



## -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from pysvnmanager.tests import *

class TestReposController(TestController):

    def test_index(self):

        # Test redirect to login pange
        res = self.app.get(url(controller='repos', action='index'))
        assert res.status == "302 Found", res.status
        assert res.location.endswith('/login'), res.location

        # Login as common user
        self.login('nobody')
        res = self.app.get(url(controller='repos', action='index'))
        assert res.status == "302 Found", res.status
        assert res.location.endswith('/security/failed'), res.location
        
        # repos admin can access repos controller(not root admin), but only with authed repos
        self.login('admin2')
        res = self.app.get(url(controller='repos', action='index'))
        assert res.status == "200 OK", res.status

        # ??? Repos admin can or can not manage hooks for his/her repos ???
        self.login('admin2')
        res = self.app.get(url(controller='repos', action='init_repos_list'))
        assert """id[0]="...";name[0]="Please choose...";
total=1;""" in res.body, res.body

        # Login as superuser
        self.login('root')
        res = self.app.get(url(controller='repos', action='index'))
        assert res.status == "200 OK", res.status
        assert """<div id="installed_hook_form_contents"></div>""" in res.body, res.body[:100]

    def test_init_repos_list(self):
        # Login as superuser
        self.login('root')
        res = self.app.get(url(controller='repos', action="init_repos_list"))
        assert res.status == "200 OK", res.status
        assert """id[0]="...";name[0]="Please choose...";
id[1]="repos3";name[1]="repos3";
id[2]="project1";name[2]="project1 (!)";
id[3]="project2";name[3]="project2 (!)";
total=4;""" in res.body, res.body[:"200 OK"]
    
    def test_get_plugin_list(self):
        self.login('root')
        params = {
                  'select':'project1', 
                  }
        res = self.app.get(url(controller='repos', action="get_plugin_list"), params)
        assert res.status == "200 OK", res.status
        assert "CaseInsensitive" in res.body, res.body
        assert "EolStyleCheck" in res.body, res.body
        assert "Please choose..." in res.body, res.body
    
    def test_get_installed_hook_form(self):
        self.login('root')
        params = {
                  'select':'project1', 
                  }
        res = self.app.get(url(controller='repos', action="get_installed_hook_form"), params)
        assert res.status == "200 OK", res.status
        assert "" == res.body, res.body

    def test_get_hook_setting_form(self):
        self.login('root')
        params = {
                  'repos':'project1', 
                  'plugin':'CaseInsensitive',
                  }
        res = self.app.get(url(controller='repos', action="get_hook_setting_form"), params)
        assert res.status == "200 OK", res.status
        assert "A pre-commit hook to detect case-insensitive filename clashes." in res.body, res.body
    
    def test_install_uninstall_hook(self):
        self.login('root')
        params = {
                  '_repos':'project1',
                  '_plugin':'CaseInsensitiveXXX',
                  }
        res = self.app.get(url(controller='repos', action="setup_hook"), params)
        assert res.status == "200 OK", res.status
        assert "Apply plugin 'CaseInsensitiveXXX' on 'project1' Failed" in res.body, res.body

        self.login('root')
        params = {
                  'select':'project1', 
                  }
        res = self.app.get(url(controller='repos', action="get_plugin_list"), params)
        assert res.status == "200 OK", res.status
        assert "CaseInsensitive" in res.body, res.body
        assert "EolStyleCheck" in res.body, res.body
        assert "Please choose..." in res.body, res.body

        self.login('root')
        params = {
                  '_repos':'project1',
                  '_plugin':'CaseInsensitive',
                  }
        res = self.app.get(url(controller='repos', action="setup_hook"), params)
        assert res.status == "200 OK", res.status
        assert "<div class='info'>Apply plugin 'CaseInsensitive' on 'project1' success.</div>" == res.body, res.body

        self.login('root')
        params = {
                  'select':'project1', 
                  }
        res = self.app.get(url(controller='repos', action="get_plugin_list"), params)
        assert res.status == "200 OK", res.status
        assert "CaseInsensitive" not in res.body, res.body
        assert "EolStyleCheck" in res.body, res.body
        assert "Please choose..." in res.body, res.body

        self.login('root')
        params = {
                  '_repos':'project1',
                  '_plugin':'EolStyleCheck',
                  }
        res = self.app.get(url(controller='repos', action="setup_hook"), params)
        assert res.status == "200 OK", res.status
        assert """<div class='info'>Apply plugin 'EolStyleCheck' on 'project1' success.</div>""" == res.body, res.body

        self.login('root')
        params = {
                  'select':'project1', 
                  }
        res = self.app.get(url(controller='repos', action="get_plugin_list"), params)
        assert res.status == "200 OK", res.status
        assert "CaseInsensitive" not in res.body, res.body
        assert "EolStyleCheck" not in res.body, res.body
        assert "Please choose..." in res.body, res.body

        self.login('root')
        params = {
                  '_repos':'project1',
                  'pluginid_0':'CaseInsensitive',
                  'pluginid_1':'EolStyleCheck',
                  }
        res = self.app.get(url(controller='repos', action="uninstall_hook"), params)
        assert res.status == "200 OK", res.status
        assert """<div class='info'>Delete plugin 'CaseInsensitive, EolStyleCheck' on 'project1' success.</div>""" == res.body, res.body

        self.login('root')
        params = {
                  'select':'project1', 
                  }
        res = self.app.get(url(controller='repos', action="get_plugin_list"), params)
        assert res.status == "200 OK", res.status
        assert "CaseInsensitive" in res.body, res.body
        assert "EolStyleCheck" in res.body, res.body
        assert "Please choose..." in res.body, res.body

        
    



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# Copyright (C) 2008 OpenSourceXpress Ltd. (http://www.ossxp.com)
# Author: Jiang Xin
# Contact: http://www.ossxp.com
#          http://www.worldhello.net
#          http://moinmo.in/JiangXin
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

from __future__ import division

import os
import sys
import re
import math

import logging
log = logging.getLogger(__name__)

sys.path.insert(0,os.path.dirname(os.path.dirname(os.path.dirname(os.path.abspath(__file__)))))
from pysvnmanager.lib.text import to_unicode, to_utf8

#reload(sys) # in Python2.5, method sys.setdefaultencoding 
            #will be delete after initialize. we need reload it.
#sys.setdefaultencoding('utf-8')

CMD_CI="RCSINIT= ci"
CMD_CO="RCSINIT= co"
CMD_RCS="RCSINIT= rcs"
CMD_RLOG="RCSINIT= rlog"
CMD_RCSDIFF="RCSINIT= rcsdiff"

def is_rcs_exist(wcfile):
    wcpath = os.path.dirname(os.path.abspath(wcfile))
    if os.path.isdir(wcpath+'/RCS'):
        rcsfile = wcpath+'/RCS/'+os.path.basename(wcfile)+',v'
    else:
        rcsfile = wcfile+',v'
    return os.path.exists(rcsfile)

def backup(wcfile, comment='', user=''):
    if not wcfile:
        raise Exception, "working copy file is not given."
    if not user:
        user = "pySvnManager"
    if isinstance(comment, (list, tuple)):
        comment = '\n'.join(comment)
    if not comment:
        comment = "no message."
    
    wcfile = to_utf8(wcfile)
    comment = to_utf8(comment)
    user = to_utf8(user, escape=True)
    
    cmd = []
    if not is_rcs_exist(wcfile):
        # -l : lock mode, make wcfile writable
        cmd.append('%(cmd)s -i -q -l -t-"%(msg)s" -w"%(user)s" "%(file)s" 2>&1' % \
                {'cmd':CMD_CI, "file":wcfile, "msg":comment, "user":user})
        # -U : set locking to no-strict.
        cmd.append('%(cmd)s -U -u -M -q "%(file)s"' % {'cmd':CMD_RCS, "file":wcfile})
    else:
        # Warning: w/o -l or -u option, wcfile will be removed after checkin.
        # -l makes wcfile writable;
        # -u : wcfile is not writable unless rcsfile is set to no-strict locking.
        cmd.append('%(cmd)s -q -u -m"%(msg)s" -w"%(user)s" "%(file)s" 2>&1' % \
                {'cmd':CMD_CI, "file":wcfile, "msg":comment, "user":user})

    for i in cmd:
        log.debug("Command: "+i)
        try:
            buff = os.popen(i).read().strip()
        except Exception, e:
            raise
        else:
            if buff:
                os.system('%(cmd)s -U -u -M -q "%(file)s"' % {'cmd':CMD_RCS, "file":wcfile})
                raise Exception, "Error Message: %s\n" % buff

def restore(wcfile, revision=""):
    if not wcfile or not is_rcs_exist(wcfile):
        return
    opts = ""
    if revision:
        # -uRev : checkout without lock revision.
        # unlock wcfile is readonly, unless no-strict mode is set.
        opts = "-u%s" % revision

    cmd = '%(cmd)s %(opts)s -q -f "%(file)s" 2>&1' % {'cmd':CMD_CO, "opts":opts, "file":wcfile }
    buff = os.popen(cmd).read().strip()
    if buff:
        raise Exception, "Command: %s\nError Message: %s\n" % (to_unicode(cmd), to_unicode(buff))


def cat(wcfile, revision=""):
    if not wcfile or not is_rcs_exist(wcfile):
        return ""
    opts = "-p"
    if revision:
        # -pRev : cat rather then checkout
        opts = "-p%s" % revision

    cmd = '%(cmd)s %(opts)s -q "%(file)s"' % {'cmd':CMD_CO, "opts":opts, "file":wcfile }
    buff = os.popen(cmd).read().strip()
    return to_unicode(buff)

def differ(filename, rev1="", rev2=""):
    filename=to_utf8(filename)
    opts=""
    if rev1 and rev2:
        opts="-r%s -r%s" % (rev1, rev2)
    elif rev1 or rev2:
        opts="-r%s%s" % (rev1, rev2)
        
    cmd = '%(cmd)s %(opts)s -u -q "%(file)s"' % {'cmd':CMD_RCSDIFF, 'opts':opts, 'file':filename}
    log.debug('Command: '+cmd)
    buff = os.popen(cmd).read()
    return to_unicode(buff)

class RcsLog(object):
    
    def __init__(self, filename):
        assert filename and isinstance(filename, basestring)
        self.__file = filename
        self.__log_per_page = 10
        self.p = {}
        self.p['rcs'] = re.compile(r'^RCS file:\s*(.*)$', re.M)
        self.p['head'] = re.compile(r'^head:\s*(.*)$', re.M)
        self.p['total'] = re.compile(r'^total revisions:\s*(.*)$', re.M)
        self.p['revision'] = re.compile(r'^revision\s+([^\s]*)', re.M)
        self.p['date'] = re.compile(r'^date:\s+(.*);\s+author:', re.M)
        self.p['author'] = re.compile(r'^date:.*;\s+author:\s*(.*?);', re.M)
        self.reload()
    
    def __get_page_count(self):
        if self.__total == 0:
            count = 0
        elif self.__total <= self.__log_per_page:
            count = 1
        else:
            # show last record on every page.
            #count = int( math.ceil( (self.__total -1 ) / self.__log_per_page ) )
            count = int( math.ceil( (self.__total -2 ) / (self.__log_per_page - 1) ) )

        return count
    
    total_page = property(__get_page_count)

    def __get_log_count(self):
        return self.__total
    
    total = property(__get_log_count)
    
    def __get_rcsfile(self):
        return self.__rcsfile
    
    rcsfile = property(__get_rcsfile)
        
    def __get_head(self):
        return self.__head
    
    head = property(__get_head)
        
    def __get_log_per_page(self):
        return self.__log_per_page
    
    def __set_log_per_page(self, count):
        count = int(count)
        if count < 2:
            count = 2
        self.__log_per_page = count

    log_per_page = property(__get_log_per_page, __set_log_per_page)
    
    def reload(self):
        cmd = '%(cmd)s -h -N "%(file)s"' % {'cmd':CMD_RLOG, 'file':self.__file}
        buff = os.popen(cmd).read().strip()
        # RCS file: 1,v
        m = self.p['rcs'].search(buff)
        if m:
            self.__rcsfile = m.group(1)
        else:
            self.__rcsfile = ""

        # head: 1.XX
        m = self.p['head'].search(buff)
        if m:
            self.__head = m.group(1)
        else:
            self.__head = ""
            
        # total revisions: XX
        m = self.p['total'].search(buff)
        if m:
            self.__total = int(m.group(1))
        else:
            self.__total = 0
    
    def get_page_logs(self, pagenum):
        total_page = self.total_page
        
        if total_page ==0:
            return []

        if pagenum<=0:
            pagenum=1
        elif pagenum>total_page:
            pagenum=total_page
        
        heads = self.__head.rsplit('.',1)
        rev0=int(heads[1])
        
        rev2 = rev0 - ((pagenum-1) * (self.__log_per_page-1)) -1
        if rev2<1: rev2=1
        rev1 = rev2 - self.__log_per_page + 1
        if rev1<1:
            rev1=1

        rev0 = self.__head
        rev1 = "%s.%d" % (heads[0], rev1)
        rev2 = "%s.%d" % (heads[0], rev2)

        self.get_logs(rev1, rev2, rev0)
        return self.revs
    
    def get_logs(self, rev1="", rev2="", rev3=""):
        self.revs=[]
        opts=""
        if not rev1:
            if not rev2:
                opts=""
            else:
                opts="-r:%s" % rev2
        else:
            if not rev2:
                opts="-r%s:" % rev1
            else:
                opts="-r%s:%s" % (rev1, rev2)

        if rev3:
            opts="%s,%s" % (opts, rev3)

        cmd = '%(cmd)s %(opts)s -N "%(file)s"' % {'cmd':CMD_RLOG, 'opts':opts, 'file':self.__file}
        log.debug('Command: '+cmd)
        buff = os.popen(cmd).read().strip().rstrip('=').rstrip()
        
        while True:
            pos = buff.rfind('\n'+'-'*28+'\n')
            if pos==-1:
                break
            match = buff[pos+30:]
            buff=buff[:pos]

            lines=match.split('\n')
            if len(lines)<3:
                log.error("wrong rcs format: %s" % match)
                continue

            # revision 1.XX    locked by: XXX;
            m = self.p['revision'].search(lines[0])
            commit_revision = ""
            if m:
                commit_revision = m.group(1)
            else:
                log.error("not find revision in line: %s" % lines[1])
                continue
                        
            # date: YYYY/MM/DD hh:mm:ss;  author: XX;  ...
            m = self.p['date'].search(lines[1])
            commit_time = ""
            if m:
                commit_time = to_unicode(m.group(1))
            else:
                log.error("not find date in line: %s" % lines[1])
                continue

            # date: YYYY/MM/DD hh:mm:ss;  author: XX;  ...
            m = self.p['author'].search(lines[1])
            commit_author = ""
            if m:
                commit_author = to_unicode(eval("'%s'" % m.group(1)))
            else:
                log.error("not find author in line: %s" % lines[1])
                continue
            
            # logs...
            commit_log = to_unicode('\n'.join(lines[2:]))
            
            self.revs.append({'revision':commit_revision,
                              'date':commit_time,
                              'author':commit_author,
                              'log':commit_log})
        
        return self.revs
    
    def cat(self, revision=""):
        return cat(self.__file, revision)
    
    def differ(self, rev1="", rev2=""):
        return differ(self.__file, rev1, rev2)
    
    def restore(self, revision):
        return restore(self.__file, revision)

    def backup(self, comment='', user=''):
        return backup(self.__file, comment, user)
